'''
'''
# !/usr/bin/python3

import os, sys, time, re, traceback, platform
from PyQt5 import QtGui, QtCore, QtWidgets
import KuttyPyLib
import socket

from utilities.templates import ui_layoutplus as layout
from utilities import dio, REGISTERS, uploader, syntax
from utilities import texteditor
import constants
import inspect

from functools import partial
from collections import OrderedDict


# translation stuff
def translate(lang=None):
	global app, t, t1
	if lang is None:
		lang = QtCore.QLocale.system().name()
	t = QtCore.QTranslator()
	t.load("lang/" + lang, os.path.dirname(__file__))
	app.installTranslator(t)
	t1 = QtCore.QTranslator()
	t1.load("qt_" + lang,
	        QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.TranslationsPath))
	app.installTranslator(t1)


class myTimer():
	def __init__(self, interval):
		self.interval = interval
		self.reset()

	def reset(self):
		self.timeout = time.time() + self.interval

	def ready(self):
		T = time.time()
		dt = T - self.timeout
		if dt > 0:  # timeout is ahead of current time
			# if self.interval>5:print('reset',self.timeout,dt)
			self.timeout = T - dt % self.interval + self.interval
			# if self.interval>5:print(self.timeout)
			return True
		return False

	def progress(self):
		return 100 * (self.interval - self.timeout + time.time()) / (self.interval)


class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
	p = None
	ports = ['A', 'B', 'C', 'D']
	logThis = QtCore.pyqtSignal(str)
	showStatusSignal = QtCore.pyqtSignal(str, bool)
	serverSignal = QtCore.pyqtSignal(str)
	logThisPlain = QtCore.pyqtSignal(bytes)
	codeOutput = QtCore.pyqtSignal(str, str)
	serialGaugeSignal = QtCore.pyqtSignal(bytes)
	serialGaugeConvert = 'bytes'
	serialStream = b''

	def __init__(self, parent=None, **kwargs):
		super(AppWindow, self).__init__(parent)

		self.local_ip = ''
		self.setupUi(self)
		self.compile_thread = None
		self.VERSION = REGISTERS.VERSION_ATMEGA32  # This needs to be dynamically changed when hardware is connected
		self.SPECIALS = REGISTERS.VERSIONS[self.VERSION]['SPECIALS']
		self.REGISTERS = REGISTERS.VERSIONS[self.VERSION]['REGISTERS']
		self.EXAMPLES_DIR = REGISTERS.VERSIONS[self.VERSION]['examples directory']

		self.docks = [self.padock, self.pbdock, self.pcdock, self.pddock]
		self.sensorList = []
		self.controllerList = []
		self.monitoring = True
		self.logRegisters = True
		self.userHexRunning = False
		self.uploadingHex = False
		self.autoUpdateUserRegisters = False
		self.CFile = None  # '~/kuttyPy.c'
		self.ipy = None
		self.defaultDirectory = path["examples"]
		self.serverActive = False
		examples = [a for a in os.listdir(os.path.join(path["examples"], self.EXAMPLES_DIR)) if
		            ('.py' in a) and a != 'kuttyPy.py']  # .py files except the library
		self.exampleList.addItems(examples)
		blinkindex = self.exampleList.findText('blink.py')
		if blinkindex != -1:  # default example. blink.py present in examples directory
			self.exampleList.setCurrentIndex(blinkindex)

		self.serialGuageButton.hide()  # Hide the serial guage button
		# Define some keyboard shortcuts for ease of use
		self.shortcutActions = {}
		self.shortcuts = {"f": partial(self.setLanguage, 'fr_FR'), "e": partial(self.setLanguage, 'en_IN'),
		                  "m": partial(self.setLanguage, 'ml_IN'), "Ctrl+S": self.saveFile, "Ctrl+O": self.openFile,
		                  "Ctrl+Shift+S": self.saveAs, "Ctrl+N": self.addSourceTab}
		for a in self.shortcuts:
			shortcut = QtWidgets.QShortcut(QtGui.QKeySequence(a), self)
			shortcut.activated.connect(self.shortcuts[a])
			self.shortcutActions[a] = shortcut
		######## PYTHON CODE
		self.codeThread = QtCore.QThread()
		self.codeEval = self.codeObject(self.REGISTERS)
		self.codeEval.moveToThread(self.codeThread)
		self.codeEval.finished.connect(self.codeThread.quit)
		self.codeEval.logThis.connect(self.appendLog)  # Connect to the log window
		self.logThis.connect(self.appendLog)  # Connect to the log window
		self.logThisPlain.connect(self.appendLogPlain)  # Connect to the log window
		self.serialGaugeSignal.connect(self.setSerialgauge)

		self.codeThread.started.connect(self.codeEval.execute)
		self.codeThread.finished.connect(self.codeFinished)

		########## C CODE EDITOR SYNTAX HIGHLIGHTER
		self.code_highlighters = []
		####### C CODE EDITOR #########
		self.codingTabs.tabCloseRequested.connect(self.closeCTab)
		self.codingTabs.tabBarClicked.connect(self.CTabChanged)
		self.addFileMenu()
		self.listTab = None
		self.mapTab = None
		self.hexTab = None

		self.activeEditor = None
		self.activeSourceTab = None
		self.sourceTabs = {}
		self.addSourceTab()

		######### C CODE UPLOADER
		self.uploadThread = QtCore.QThread()
		self.UploadObject = self.uploadObject()
		self.UploadObject.moveToThread(self.uploadThread)
		self.UploadObject.finished.connect(self.uploadThread.quit)
		self.UploadObject.logThis.connect(self.appendLog)  # Connect to the log window
		self.UploadObject.resultSignal.connect(self.codeOutput)  # Load .lst, .hex , .map files
		self.UploadObject.logThisPlain.connect(self.appendLogPlain)  # Connect to the log window. add plain text
		self.logThis.connect(self.appendLog)  # Connect to the log window

		self.uploadThread.started.connect(self.UploadObject.execute)
		self.uploadThread.finished.connect(self.codeFinished)

		self.commandQ = []
		self.btns = {}
		self.registers = []
		self.addPins()

		self.statusBar = self.statusBar()
		self.makeBottomMenu()

		global app

		self.initializeCommunications()
		self.pending = {
			'status': myTimer(constants.STATUS_UPDATE_INTERVAL),
			'update': myTimer(constants.AUTOUPDATE_INTERVAL),
		}

		serialgaugeoptions = {'name': 'Serial Monitor', 'init': print, 'read': None,
		                      'fields': ['Value'],
		                      'min': [0],
		                      'max': [1000],
		                      'config': [{
			                      'name': 'Data Type',
			                      'options': ['byte', 'ASCII'],
			                      'function': self.configSerialGauge
		                      }
		                      ]}
		self.serialGauge = dio.DIOSENSOR(self, serialgaugeoptions)

		self.startTime = time.time()
		self.timer = QtCore.QTimer()
		self.timer.timeout.connect(self.updateEverything)
		self.timer.start(20)

		# Auto-Detector
		self.shortlist = KuttyPyLib.getFreePorts()

	def activateCompileServer(self):
		if self.serverActive: #Stop it
			if self.compile_thread is not None:
				self.compile_thread.terminate()
				self.compile_thread.wait()
				print('quit compile_thread')
				self.compile_thread_button.setParent(None)
				self.showStatus("Compiler Stopped ", False)
				return
			self.serverActive = False
			self.compile_thread_button.setText("Start Online Compiler")
		else:
			s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
			s.settimeout(0.1)  # Set a timeout to avoid blocking indefinitely
			s.connect(("8.8.8.8", 80))  # Connect to a public IP address
			self.local_ip = s.getsockname()[0]

			from utilities.compile_server import create_server
			self.compile_thread = create_server(self.showStatusSignal, self.serverSignal, path["kpy"], self.local_ip)
			self.showStatusSignal.connect(self.showStatus)
			self.serverSignal.connect(self.showServerStatus)
			s.close()
			self.showStatus("Compiler Active at " + self.local_ip, False)
			self.compile_thread_button.setText("Stop Server")
			self.serverActive = True

	def addFileMenu(self):
		self.myFileMenuBar = QtWidgets.QMenuBar(self.fileMenuButton)
		codeMenu = self.myFileMenuBar.addMenu('File:')

		newFileAction = QtWidgets.QAction('New File', self)
		newFileAction.triggered.connect(self.addSourceTab)
		codeMenu.addAction(newFileAction)

		openFileAction = QtWidgets.QAction('Open File', self)
		openFileAction.triggered.connect(self.openFile)
		openIcon = QtGui.QIcon()
		openIcon.addPixmap(QtGui.QPixmap(":/control/document-open.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		openFileAction.setIcon(openIcon)
		codeMenu.addAction(openFileAction)

		saveFileAction = QtWidgets.QAction('Save File', self)
		saveFileAction.triggered.connect(self.saveFile)
		saveIcon = QtGui.QIcon()
		saveIcon.addPixmap(QtGui.QPixmap(":/control/saveall.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		saveFileAction.setIcon(saveIcon)
		codeMenu.addAction(saveFileAction)

		saveAsFileAction = QtWidgets.QAction('Save As', self)
		saveAsFileAction.triggered.connect(self.saveAs)
		saveAsFileAction.setIcon(saveIcon)
		codeMenu.addAction(saveAsFileAction)

		exitAction = QtWidgets.QAction('Exit', self)
		exitAction.triggered.connect(QtWidgets.qApp.quit)
		codeMenu.addAction(exitAction)

	def closeCTab(self, index):
		print('Close Tab', index)
		widget = self.codingTabs.widget(index)
		sourceTabClosed = False
		if widget == self.mapTab:
			print('closing map tab')
			self.mapTab = None
		elif widget == self.hexTab:
			print('closing hex tab')
			self.hexTab = None
		elif widget == self.listTab:
			print('closing list tab')
			self.listTab = None
		elif widget in self.sourceTabs:
			if len(self.sourceTabs) == 1:
				print("last tab. won't close.")
				return
			else:
				print('closing source tab', widget.objectName())
				self.sourceTabs.pop(widget)
				sourceTabClosed = True

		self.codingTabs.removeTab(index)
		if sourceTabClosed:  # Source Tab closed. Re-assign active source tab
			self.activeSourceTab = list(self.sourceTabs.keys())[0]
			self.activeEditor = self.sourceTabs[self.activeSourceTab][0]
			self.CFile = self.sourceTabs[self.activeSourceTab][1]
			self.codingTabs.setCurrentIndex(self.codingTabs.indexOf(self.activeSourceTab))
			print('New Source Tab:', self.getActiveFilename, self.CFile)

	def getActiveFilename(self):
		self.codingTabs.tabText(self.codingTabs.indexOf(self.activeSourceTab))

	def addSourceTab(self):
		sourceTab = QtWidgets.QWidget()
		sourceTab.setObjectName("sourceTab")
		horizontalLayout_3 = QtWidgets.QHBoxLayout(sourceTab)
		horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
		horizontalLayout_3.setSpacing(0)
		horizontalLayout_3.setObjectName("horizontalLayout_3")
		editor = texteditor.myTextEditor(sourceTab, self.codingTabs)
		font = QtGui.QFont()
		font.setFamily("Ubuntu Mono")
		font.setPointSize(12)
		editor.setFont(font)
		editor.setObjectName("editor")
		editor.setTabChangesFocus(False)
		horizontalLayout_3.addWidget(editor)
		self.codingTabs.addTab(sourceTab, "")
		self.sourceTabs[sourceTab] = [editor, None]
		self.codingTabs.setCurrentIndex(self.codingTabs.indexOf(sourceTab))
		self.codingTabs.setTabText(self.codingTabs.indexOf(sourceTab), 'new.c')
		self.activeSourceTab = sourceTab
		self.activeEditor = editor
		self.code_highlighters.append(syntax.PythonHighlighter(editor.document()))

	def CTabChanged(self, index):
		widget = self.codingTabs.widget(index)
		if widget in self.sourceTabs:
			self.activeSourceTab = widget
			self.activeEditor = self.sourceTabs[widget][0]
			self.CFile = self.sourceTabs[widget][1]
			print('Change Tab', index, self.codingTabs.tabText(index), self.CFile)

	def newRegister(self):
		reg = dio.REGEDIT(self.commandQ)
		# self.registerLayout.addWidget(reg)
		# self.registers.append(reg)

		# TODO: Convert layout to listwidget to enable re-ordering
		regItem = QtWidgets.QListWidgetItem()
		regItem.setSizeHint(QtCore.QSize(200, 40))
		self.registerList.addItem(regItem)
		self.registerList.setItemWidget(regItem, reg)
		self.registers.append(regItem)

	def addPins(self):
		for port, dock in zip(self.ports, [self.palayout, self.pblayout, self.pclayout, self.pdlayout]):
			checkbox = dio.REGVALS(port)
			dock.addWidget(checkbox)
			self.btns[port] = checkbox

			seq = range(7, -1, -1)
			if port == 'C': seq = reversed(seq)  # PORTC pins are ordered top to bottom
			for a in seq:
				name = 'P' + port + str(a)
				checkbox = dio.widget(name, self.commandQ, extra=self.SPECIALS.get(name, ''))
				dock.addWidget(checkbox)
				self.btns[name] = checkbox

	def tabChanged(self, index):
		if self.userHexRunning:  # Firmware was running. Stop it.
			self.launchFirmwareButton.setChecked(False)
			self.jumpToApplication(False)

		if index != 0:  # examples/editor tab. disable monitoring
			self.monitoring = False
			for a in self.docks:
				a.hide()
		else:  # Playground . enable monitoring and control.
			self.monitoring = True
			self.autoRefreshUserRegisters.setChecked(False)
			self.userRegistersAutoRefresh(False)
			self.setLogType('playground')
			for a in self.docks:
				a.show()

	def setLogType(self, tp):
		self.log.clear()
		if tp == 'playground':
			self.logLabel.setText("Monitor registers being read and set during each operation")
		elif tp == 'avr':
			self.logLabel.setText("Avr-Gcc compile/upload messages")
		elif tp == 'monitor':
			self.logLabel.setText("Monitor data input from the serial port")
		elif tp == 'python':
			self.logLabel.setText("Python Code operations monitor")

	################USER CODE SECTION####################
	def codeOutput(self, filetype, filename):
		self.openCodeBreakupTab(filetype, filename)
		try:
			infile = open(filename, 'r')
			if filetype == 'list':
				self.listEditor.setPlainText(infile.read())
			if filetype == 'map':
				self.mapEditor.setPlainText(infile.read())
			if filetype == 'hex':
				self.hexEditor.setPlainText(infile.read())
			infile.close()
		except Exception as e:
			print(e)

	def openCodeBreakupTab(self, filetype, filename):
		tab = None
		if filetype == 'list':
			if self.listTab is None:
				self.openListTab()
			tab = self.listTab
		elif filetype == 'map':
			if self.mapTab is None:
				self.openMapTab()
			tab = self.mapTab
		elif filetype == 'hex':
			if self.hexTab is None:
				self.openHexTab()
			tab = self.hexTab

		if tab != None:
			fname = filename.split
			self.codingTabs.setTabText(self.codingTabs.indexOf(tab), filetype + ":" + os.path.basename(filename))

	def openListTab(self):
		self.listTab = QtWidgets.QWidget()
		self.listTab.setObjectName("listTab")
		self.verticalLayout = QtWidgets.QVBoxLayout(self.listTab)
		self.verticalLayout.setContentsMargins(0, 0, 0, 0)
		self.verticalLayout.setSpacing(0)
		self.verticalLayout.setObjectName("verticalLayout")
		self.listEditor = QtWidgets.QTextBrowser(self.listTab)
		self.listEditor.setObjectName("listEditor")
		self.verticalLayout.addWidget(self.listEditor)
		self.codingTabs.addTab(self.listTab, "")

	def openMapTab(self):
		self.mapTab = QtWidgets.QWidget()
		self.mapTab.setObjectName("mapTab")
		self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.mapTab)
		self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
		self.verticalLayout_3.setSpacing(0)
		self.verticalLayout_3.setObjectName("verticalLayout_3")
		self.mapEditor = QtWidgets.QTextBrowser(self.mapTab)
		self.mapEditor.setObjectName("mapEditor")
		self.verticalLayout_3.addWidget(self.mapEditor)
		self.codingTabs.addTab(self.mapTab, "")

	def openHexTab(self):
		self.hexTab = QtWidgets.QWidget()
		self.hexTab.setObjectName("hexTab")
		self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.hexTab)
		self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
		self.verticalLayout_2.setSpacing(0)
		self.verticalLayout_2.setObjectName("verticalLayout_2")
		self.hexEditor = QtWidgets.QTextBrowser(self.hexTab)
		self.hexEditor.setObjectName("hexEditor")
		self.verticalLayout_2.addWidget(self.hexEditor)
		self.codingTabs.addTab(self.hexTab, "")

	class codeObject(QtCore.QObject):
		finished = QtCore.pyqtSignal()
		logThis = QtCore.pyqtSignal(str)
		code = ''

		def __init__(self, REGISTERS):
			super(AppWindow.codeObject, self).__init__()
			self.PORTMAP = {v: k for k, v in REGISTERS.items()}  # Lookup port name based on number
			self.compiled = ''
			self.SR = None
			self.GR = None
			self.evalGlobals = {}

			self.evalGlobals['getReg'] = self.getReg
			self.evalGlobals['setReg'] = self.setReg
			self.evalGlobals['print'] = self.printer

		def setCode(self, code, **kwargs):
			try:
				self.compiled = compile(code.encode(), '<string>', mode='exec')
			except SyntaxError as err:
				error_class = err.__class__.__name__
				detail = err.args[0]
				line_number = err.lineno
				return '''<span style="color:red;">%s at line %d : %s</span>''' % (error_class, line_number, detail)
			except Exception as err:
				error_class = err.__class__.__name__
				detail = err.args[0]
				cl, exc, tb = sys.exc_info()
				line_number = traceback.extract_tb(tb)[-1][1]
				return '''<span style="color:red;">%s at line %d: %s</span>''' % (error_class, line_number, detail)

			self.SR = kwargs.get('setReg')
			self.GR = kwargs.get('getReg')
			self.evalGlobals = kwargs
			self.evalGlobals['getReg'] = self.getReg  # Overwrite these three. They will be wrapped.
			self.evalGlobals['setReg'] = self.setReg
			self.evalGlobals['print'] = self.printer
			return ''

		def printer(self, *args):
			self.logThis.emit('''<span style="color:cyan;">%s</span>''' % (' '.join([str(a) for a in args])))

		def setReg(self, reg, value):
			html = u'''<pre><span>W\u2193</span>{0:s}\t{1:d}\t0x{1:02x} / 0b{1:08b}</pre>'''.format(
				self.PORTMAP.get(reg, ''), value)
			self.logThis.emit(html)
			self.SR(reg, value)

		def getReg(self, reg):
			value = self.GR(reg)
			html = u'''<pre><span>R\u2191</span>{0:s}\t{1:d}\t0x{1:02x} / 0b{1:08b}</pre>'''.format(
				self.PORTMAP.get(reg, ''), value)
			self.logThis.emit(html)
			return value

		def execute(self):
			# old = sys.stdout
			# olde = sys.stderr
			# sys.stdout = self.toLog(self.logThis)
			# sys.stderr = self.toLog(self.logThis)
			try:
				exec(self.compiled, {}, self.evalGlobals)
			except SyntaxError as err:
				error_class = err.__class__.__name__
				detail = err.args[0]
				line_number = err.lineno
				self.logThis.emit(
					'''<span style="color:red;">%s at line %d : %s</span>''' % (error_class, line_number, detail))
			except Exception as err:
				error_class = err.__class__.__name__
				detail = err.args[0]
				cl, exc, tb = sys.exc_info()
				line_number = traceback.extract_tb(tb)[-1][1]
				self.logThis.emit(
					'''<span style="color:red;">%s at line %d: %s</span>''' % (error_class, line_number, detail))
			# sys.stdout = old
			# sys.stderr = olde
			self.logThis.emit("Finished executing user code")
			self.finished.emit()

	def runCode(self):
		if self.codeThread.isRunning():
			print('one code is already running')
			return

		self.setLogType('python')
		self.log.clear()  # clear the log window
		self.log.setText('''<span style="color:green;">----------User Code Started-----------</span>''')
		kwargs = {}
		for a in dir(self.p):
			attr = getattr(self.p, a)
			if inspect.ismethod(attr) and a[:2] != '__':
				kwargs[a] = attr

		compilemsg = self.codeEval.setCode('{0:s}'.format(self.userCode.toPlainText()), **kwargs)
		if len(compilemsg):
			self.log.append(compilemsg)
			return
		self.codeThread.start()

		self.userCode.setStyleSheet("border: 3px dashed #53ffff;")
		self.tabs.setTabEnabled(0, False)

	def codeFinished(self):
		print('finished')
		self.tabs.setTabEnabled(0, True)
		self.userCode.setStyleSheet("")
		self.uploadingHex = False

	def abort(self):
		if self.codeThread.isRunning():
			self.log.append(
				'''<span style="color:red;">----------Kill Signal(Doesn't work yet. Restart the application)-----------</span>''')
			self.codeThread.quit()
			self.codeThread.terminate()
			del self.codeThread
			self.codeThread = QtCore.QThread()
			self.codeEval = self.codeObject(self.REGISTERS)
			self.codeEval.moveToThread(self.codeThread)
			self.codeEval.finished.connect(self.codeThread.quit)
			self.codeEval.logThis.connect(self.appendLog)  # Connect to the log window
			self.codeThread.started.connect(self.codeEval.execute)
			self.codeThread.finished.connect(self.codeFinished)

	def getReg(self, reg, record=True):
		val = self.p.getReg(reg)
		if record:
			self.updatedRegs[reg] = [0, val]
		return val

	def setReg(self, reg, val, record=True):
		self.p.setReg(reg, val)
		if record:
			self.updatedRegs[reg] = [1, val]
		return val

	def appendLog(self, txt):
		self.log.append(txt)

	def appendLogPlain(self, txt):
		self.log.moveCursor(QtGui.QTextCursor.End)
		self.log.insertPlainText(txt.decode('ascii'))

	def configSerialGauge(self, val):
		if val == 0:  # 'byte'
			print('Byte Mode(0-255)')
			for a in self.serialGauge.gauges:
				a.set_MaxValue(255)
			self.serialGaugeConvert = 'bytes'
		if val == 1:  # 'ascii'
			print('ASCII(0-10000)')
			for a in self.serialGauge.gauges:
				a.set_MaxValue(10000)
			self.serialGaugeConvert = 'ascii'

	def setSerialgauge(self, vals):
		if self.serialGaugeConvert == 'bytes':
			self.serialGauge.setValue([vals[0]])
		elif self.serialGaugeConvert == 'ascii':
			self.serialStream += vals
			while b'\n' in self.serialStream:
				val, _, self.serialStream = self.serialStream.partition(b'\n')
				self.serialGauge.setValue([float(val)])

	def genLog(self):
		html = '''<table border="1" align="center" cellpadding="1" cellspacing="0" style="font-family:arial,helvetica,sans-serif;font-size:9pt;">
		<tbody><tr><td colspan="4">%s</td></tr>''' % (time.ctime())
		# html+='''<tr><td style="background-color:#77cfbb;">R/W</td><td style="background-color:#77cfbb;">REGISTER</td>
		# <td style="background-color:#77cfbb;">Value</td><td style="background-color:#77cfbb;">Hex/Binary</td></tr>'''

		for a in self.updatedRegs:
			row = self.updatedRegs[a]
			html += u'''
				<tr>
					<td>{0:s}</td>
					<td>{1:s}</td>
					<td>{2:d}</td>
					<td>0b{2:08b} | 0x{2:02x}</td>
				</tr>
				'''.format(u'W \u2193' if row[0] else u'R \u2191', a, row[1])
		html += "</tbody></table>"
		self.log.setHtml(html)

	def userRegistersAutoRefresh(self, state):
		self.autoUpdateUserRegisters = state

	def updateEverything(self):
		self.locateDevices()
		if not self.checkConnectionStatus(): return
		# KuttyPy monitor has handed over control to native code. act as serial monitor/ debug window
		if self.uploadingHex:
			return

		if self.userHexRunning:
			t = self.p.fd.read(self.p.fd.in_waiting)
			if len(t):
				self.serialGaugeSignal.emit(t)
				self.logThisPlain.emit(t)
			return

		# self.setTheme('material')

		if self.codeThread.isRunning():
			return

		if self.autoUpdateUserRegisters:
			for a in range(self.registerList.count()):
				self.registerList.itemWidget(self.registerList.item(a)).execute()

		while len(self.commandQ):
			if not self.centralWidget().isEnabled():
				return
			a = self.commandQ.pop(0)
			if a[0] == 'DSTATE':  # Digital Out ['DSTATE','Pxx',state]
				pname = 'PORT' + a[1][1].upper()
				bit = int(a[1][2])
				reg = self.getReg(pname)
				reg &= ~ (1 << bit)
				if (a[2]): reg |= (1 << bit)
				self.setReg(pname, reg)
			elif a[0] == 'DTYPE':  # Digital pin I/O ['DTYPE','Pxx',state]
				pname = 'DDR' + a[1][1].upper()
				bit = int(a[1][2])
				reg = self.getReg(pname)
				reg &= ~ (1 << bit)
				if (a[2]): reg |= (1 << bit)
				self.setReg(pname, reg);
			elif a[0] == 'WRITE':  # ['WRITE','REGNAME',val]
				self.setReg(a[1], a[2]);
			elif a[0] == 'READ':  # ['READ','REGNAME',function]
				val = self.getReg(a[1])
				a[2](val)
			elif a[0] == 'CNTR1':  # ['CNTR1',output with setValue function]
				cl = self.getReg('TCNT1L', False)
				ch = self.getReg('TCNT1H', False)
				a[1].setValue(cl | (ch << 8))
			elif a[0] == 'ADC':  # ['ADC',ADMUX,output with setValue function, to log, or not to log]
				self.setReg('ADMUX', a[1], a[3])
				self.setReg('ADCSRA', 196 | 1, a[3])
				adcl = self.getReg('ADCL', a[3])
				adch = self.getReg('ADCH', a[3])
				a[2].setValue(adcl | (adch << 8))

		for a in self.sensorList:
			if a[0].isVisible():
				a[0].setValue(a[0].read())

		'''
        if self.enableLog.isChecked():
            if self.clearLog.isChecked() and len(self.updatedRegs):
                self.log.clear()
            if len(self.updatedRegs):
                self.genLog()
                self.updatedRegs = OrderedDict()
        '''
		if len(self.updatedRegs):
			self.genLog()
			self.updatedRegs = OrderedDict()

		if self.pending['status'].ready() and self.monitoring:
			val = self.p.getReg(self.getRegs[self.currentRegister][0])
			self.getRegs[self.currentRegister][1](self.getRegs[self.currentRegister][0], val)
			self.currentRegister += 1
			if self.currentRegister == len(self.getRegs):
				self.currentRegister = 0
			for a in self.ports:
				self.btns[a].setRegs(self.p.REGSTATES)

		if self.pending['update'].ready():
			pass

	# print('update')

	def updateInputs(self, port, value):
		portchar = port[3]
		for a in range(8):
			btn = self.btns['P' + portchar + str(a)]
			btn.nameIn.setChecked((value >> a) & 1)
			if portchar == 'A':  # ADC
				if btn.currentPage == 2:  # ADC Page displayed
					self.commandQ.append(['ADC', btn.ADMUX, btn, btn.logstate])
			elif type(btn) == dio.DIOCNTR and btn.currentPage == 2:  # CNTR
				self.commandQ.append(['CNTR1', btn.slider])

	def newStepperController(self):
		if self.p.connected:
			dialog = dio.DIOSTEPPER(self, total=200, device=self.p)
			dialog.launch()
			self.sensorList.append([dialog, None])

	############ I2C SENSORS #################
	def I2CScan(self):
		if self.p.connected:
			x = self.p.I2CScan()
			print('Responses from: ', x)
			for a in self.sensorList:
				a[0].setParent(None)
				a[1].setParent(None)
			self.sensorList = []
			for a in self.controllerList:
				a[0].setParent(None)
				a[1].setParent(None)
			self.controllerList = []
			for a in x:
				s = self.p.sensors.get(a, None)
				if s != None:
					btn = QtWidgets.QPushButton(s['name'] + ':' + hex(a))
					dialog = dio.DIOSENSOR(self, s)
					btn.clicked.connect(dialog.launch)
					self.sensorLayout.addWidget(btn)
					self.sensorList.append([dialog, btn])
					continue

				s = self.p.controllers.get(a, None)
				if s != None:
					btn = QtWidgets.QPushButton(s['name'] + ':' + hex(a))
					dialog = dio.DIOCONTROL(self, s)
					btn.clicked.connect(dialog.launch)
					self.sensorLayout.addWidget(btn)
					self.controllerList.append([dialog, btn])
					continue

				s = self.p.special.get(a, None)
				if s != None:
					btn = QtWidgets.QPushButton(s['name'] + ':' + hex(a))
					dialog = dio.DIOROBOT(self, s)
					btn.clicked.connect(dialog.launch)
					self.sensorLayout.addWidget(btn)
					self.controllerList.append([dialog, btn])
					continue

	def loadExample(self, filename):
		self.userCode.setPlainText(open(os.path.join(path["examples"], self.EXAMPLES_DIR, filename), "r").read())

	########################### UPLOAD HEX FILE #######################

	class uploadObject(QtCore.QObject):
		finished = QtCore.pyqtSignal()
		logThis = QtCore.pyqtSignal(str)
		resultSignal = QtCore.pyqtSignal(str, str)
		logThisPlain = QtCore.pyqtSignal(bytes)
		fname = ''
		p = None

		def __init__(self):
			super(AppWindow.uploadObject, self).__init__()

		def config(self, mode, p, fname):
			self.p = p
			self.fname = fname
			self.mode = mode

		def execute(self):
			if 'compile' in self.mode:
				try:
					import subprocess
					if self.fname[-2:] in ['.s', '.S']:
						action = 'Assembl'
					else:  # elif self.fname[-2:] in ['.c','.C']:
						action = 'Compil'
					fname = '.'.join(self.fname.split('.')[:-1])
					if self.p.version == REGISTERS.VERSION_ATMEGA32 or self.p.connected == False:  # by default, compile for ATMEGA32.
						# cmd = 'avr-gcc -Wall -O2 -mmcu=%s -o "%s" -Map "%s" "%s"' %('atmega32',fname,fname+'.map',self.fname)
						cmd = 'avr-gcc -Wall -O2 -mmcu=%s -Wl,-Map="%s" -o "%s" "%s" -lkp' % (
							'atmega32', fname + '.map', fname, self.fname)  # includes MAP
						self.logThis.emit('''<span style="color:green;">%sing for Atmega32</span>''' % (action))
					elif self.p.version == REGISTERS.VERSION_ATMEGA328P:
						cmd = 'avr-gcc -Wall -O2 -mmcu=%s -o "%s" "%s" -lkp' % ('atmega328p', fname, self.fname)
						self.logThis.emit(
							'''<span style="color:green;">%sing for Atmega328p (Nano)</span>''' % (action))
					else:
						self.logThis.emit('''<span style="color:white;">%ser UNAVAILABLE</span>''' % (action))
						return
					print(cmd)
					res = subprocess.getstatusoutput(cmd)
					if res[0] != 0:
						self.logThis.emit('''<span style="color:#d730ee;">%se Error: %s</span>''' % (
							action, res[1].replace('\n', '<br>')))
						self.finished.emit()
						return

					else:
						self.logThis.emit('''<span style="color:gray;">%s</span><br>''' % res[1])
						self.resultSignal.emit('map', fname + '.map')

					cmd = 'avr-objcopy -j .text -j .data -O ihex "%s" "%s.hex"' % (fname, fname)
					res = subprocess.getstatusoutput(cmd)
					self.logThis.emit(res[1])  # '''<span style="color:gray;">%s</span><br>'''%res[1])
					self.resultSignal.emit('hex', fname + '.hex')

					cmd = 'avr-objdump -S "%s" > "%s.lst"' % (fname, fname)
					res = subprocess.getstatusoutput(cmd)
					self.logThis.emit(res[1])  # '''<span style="color:gray;">%s</span><br>'''%res[1])
					self.logThis.emit(
						'Finished %sing: Generated Hex File' % action)  # '''<span style="color:darkgreen;">Finished %sing: Generated Hex File</span>'''%(action))
					self.resultSignal.emit('list', fname + '.lst')
				except Exception as err:
					self.logThis.emit('''<span style="color:#d730ee;">Failed to %se:%s</span>''' % str(action, err))

			if self.p.connected:
				if 'upload' in self.mode:
					try:
						if self.fname[-2:] in ['.c', '.C', '.s', '.S']:
							self.fname = self.fname[:-2] + '.hex'  # Replace .c with .hex
						self.logThis.emit('''<span style="font-size:12pt">Upload Code... Trigger Reset...</span>''')
						dude = uploader.Uploader(self.p.fd, hexfile=self.fname, logger=self.logThis)
						self.p.fd.setRTS(0);
						self.p.fd.setDTR(0);
						time.sleep(0.01);
						self.p.fd.setRTS(1);
						self.p.fd.setDTR(1);
						time.sleep(0.05)
						dude.program()
						dude.verify()
						self.p.fd.setRTS(0);
						self.p.fd.setDTR(0);
						time.sleep(0.01);
						self.p.fd.setRTS(1);
						self.p.fd.setDTR(1);
						time.sleep(0.25)
						self.p.get_version()
						self.logThis.emit('''<span style="color:darkgreen;">Finished upload</span>''')
					except Exception as err:
						print('upload error', err)
						self.p.fd.setRTS(0);
						self.p.fd.setDTR(0);
						time.sleep(0.01);
						self.p.fd.setRTS(1);
						self.p.fd.setDTR(1);
						time.sleep(0.25)
						self.p.get_version()
						self.logThis.emit('''<span style="color:#d730ee;">Failed to upload</span>''')
				# self.jumpToApplication(False) #Force a reset
			self.finished.emit()

	def uploadHex(self):
		filename = QtWidgets.QFileDialog.getOpenFileName(self, " Open a hex file to upload to your KuttyPy", "",
		                                                 "Hex Files (*.hex)")
		if len(filename[0]):
			# self.userCode.setStyleSheet("border: 3px dashed #53ffff;")
			# self.tabs.setTabEnabled(0,False)
			self.uploadingHex = True
			self.log.clear()
			self.log.setText('''<span style="color:cyan;">-- Uploading Code --</span><br>''')
			self.UploadObject.config('upload', self.p, filename[0])
			self.uploadThread.start()

	def openFile(self):
		filename = QtWidgets.QFileDialog.getOpenFileName(self, " Open a C/Asm file to edit", self.defaultDirectory,
		                                                 "C/ASM Files (*.c *.C *.s *.S);; Asm Files (*.s *.S);; C Files (*.c *.C)")
		if len(filename[0]):
			if self.CFile is not None:  # A file is altready open
				self.addSourceTab()
			self.defaultDirectory = ''
			self.filenameLabel.setText(filename[0])
			self.CFile = filename[0]
			self.sourceTabs[self.activeSourceTab][1] = self.CFile
			self.log.clear()
			infile = open(filename[0], 'r')
			self.activeEditor.setPlainText(
				infile.read())  # self.activeEditor = self.sourceTabs[self.activeSourceTab][0]
			infile.close()
			self.codingTabs.setTabText(self.codingTabs.indexOf(self.activeSourceTab), os.path.split(self.CFile)[1])
			self.log.setText('''<span style="color:cyan;">-- Opened File: %s --</span><br>''' % filename[0])

	def fontPlus(self):
		size = self.editorFont.pointSize()
		if size > 40: return
		self.editorFont.setPointSize(size + 1)
		self.editor.setFont(self.editorFont)

	def fontMinus(self):
		size = self.editorFont.pointSize()
		if size < 5: return
		self.editorFont.setPointSize(size - 1)
		self.editor.setFont(self.editorFont)

	def setFont(self, font):
		self.editorFont.setFamily(font)
		self.editor.setFont(self.editorFont)

	def compileAndUpload(self):
		self.setLogType('avr')
		self.saveFile()
		if self.CFile:
			if self.userHexRunning:
				self.launchFirmwareButton.setChecked(False)
				self.jumpToApplication(False)
			self.uploadingHex = True
			self.log.setText('''<span style="color:cyan;">-- Compiling and Uploading Code --</span><br>''')
			self.UploadObject.config('compileupload', self.p, self.CFile)
			self.uploadThread.start()

	def compile(self):
		self.setLogType('avr')
		self.saveFile()
		if self.CFile:
			self.log.setText('''<span style="color:green;">-- Compiling Code --</span><br>''')
			self.log.setText('''<span style="color:green;">-- %s --</span><br>''' % self.CFile)
			self.UploadObject.config('compile', self.p, self.CFile)
			self.uploadThread.start()

	def upload(self):
		self.setLogType('avr')
		if self.CFile:
			if self.userHexRunning:
				self.launchFirmwareButton.setChecked(False)
				self.jumpToApplication(False)
			self.uploadingHex = True
			self.log.setText('''<span style="color:green;">-- Uploading Code --</span><br>''')
			self.UploadObject.config('upload', self.p, self.CFile)
			self.uploadThread.start()

	def saveFile(self):
		if not self.CFile:
			self.CFile = self.saveAs()
		if self.CFile is not None and len(self.CFile) > 1:
			self.sourceTabs[self.activeSourceTab][1] = self.CFile
			fn = open(self.CFile, 'w')
			fn.write(self.activeEditor.toPlainText())
			fn.close()
			self.codingTabs.setTabText(self.codingTabs.indexOf(self.activeSourceTab), os.path.split(self.CFile)[1])
			self.log.setText('''<span style="color:green;">-- Saved to: %s --</span><br>''' % self.CFile)
		else:
			self.log.setText('''<span style="color:#d730ee;">-- No File Selected --</span><br>''')

	def saveAs(self):
		name = QtWidgets.QFileDialog.getSaveFileName(self, 'Save File')
		print('created new file:', name)
		if len(name) > 0 and len(name[0]) > 1:
			self.CFile = name[0]
			self.log.setText('''<span style="color:green;">-- Created new file: %s --</span><br>''' % self.CFile)
			self.codingTabs.setTabText(self.codingTabs.indexOf(self.activeSourceTab), os.path.split(self.CFile)[1])
			return self.CFile

	##############################
	def setTheme(self, theme):
		self.setStyleSheet("")
		self.setStyleSheet(open(os.path.join(path["themes"], theme + ".qss"), "r").read())

	def initializeCommunications(self, port=False):
		if self.p:
			try:
				self.p.fd.close()
			except:
				pass
		if port:
			self.p = KuttyPyLib.connect(port=port)
		else:
			self.p = KuttyPyLib.connect(autoscan=True)
		if self.p.connected:
			self.launchFirmwareButton.setChecked(False)
			self.setWindowTitle('KuttyPy Interactive Console [{0:s}]'.format(self.p.portname))
			self.updatedRegs = OrderedDict()
			self.currentRegister = 0
			self.VERSION = self.p.version
			self.SPECIALS = REGISTERS.VERSIONS[self.VERSION]['SPECIALS']
			self.REGISTERS = REGISTERS.VERSIONS[self.VERSION]['REGISTERS']
			self.EXAMPLES_DIR = REGISTERS.VERSIONS[self.VERSION]['examples directory']
			if self.p.version == REGISTERS.VERSION_ATMEGA328P:
				self.getRegs = [
					('PINB', self.updateInputs),
					('PINC', self.updateInputs),
					('PIND', self.updateInputs),
				]
			elif self.p.version == REGISTERS.VERSION_ATMEGA32:
				self.getRegs = [
					('PINA', self.updateInputs),
					('PINB', self.updateInputs),
					('PINC', self.updateInputs),
					('PIND', self.updateInputs),
				]


		else:
			self.setWindowTitle('KuttyPy Interactive Console [ Hardware not detected ]')

	def setLanguage(self, lang='fr_FR'):
		translate(lang)
		self.retranslateUi(self)

	def showSerialGauge(self):
		self.serialGauge.show()

	def jumpToApplication(self, state):
		if self.p:
			if state:
				self.serialGuageButton.show()
				self.userHexRunning = True
				self.p.fd.write(b'j')  # Skip to application (Bootloader resets)
				self.launchFirmwareButton.setText('Stop')

				for a in self.docks:
					a.setEnabled(False)
				# self.tabs.setEnabled(False)
				self.setLogType('monitor')
				self.log.setText('''<span style="color:cyan;">-- Serial Port Monitor --</span><br>''')
			# self.serialGauge.show()

			else:
				self.serialGuageButton.hide()
				self.p.fd.setRTS(0);
				self.p.fd.setDTR(0);
				time.sleep(0.01);
				self.p.fd.setRTS(1);
				self.p.fd.setDTR(1);
				time.sleep(0.2)
				while self.p.fd.in_waiting:
					self.p.fd.read()
				self.p.get_version()
				for a in self.docks:
					a.setEnabled(True)
				self.userHexRunning = False
				self.launchFirmwareButton.setText('Run')
				# self.tabs.setEnabled(True)
				self.serialGauge.hide()
		else:
			if self.isChecked():
				self.setChecked(False)

	def makeBottomMenu(self):
		try:
			self.pushbutton.setParent(None)
		except:
			pass
		self.pushbutton = QtWidgets.QPushButton('Menu')
		self.pushbutton.setStyleSheet("height: 13px;padding:3px;color: #7c7;")
		menu = QtWidgets.QMenu()

		menu.addAction('Save Window as Svg', self.exportSvg)
		menu.addAction('Open Stepper Controller', self.newStepperController)

		menu.addAction('Upload Hex File', self.uploadHex)

		# Theme
		'''
        self.themeAction = QtWidgets.QWidgetAction(menu)
        themes = [a.split('.qss')[0] for a in os.listdir(path["themes"]) if '.qss' in a]
        self.themeBox = QtWidgets.QComboBox(); self.themeBox.addItems(themes)
        self.themeBox.currentIndexChanged['QString'].connect(self.setTheme)
        self.themeAction.setDefaultWidget(self.themeBox)
        menu.addAction(self.themeAction)
        '''
		themes = [a.split('.qss')[0] for a in os.listdir(path["themes"]) if '.qss' in a]
		self.themeBox = QtWidgets.QComboBox();
		self.themeBox.addItems(themes)
		self.themeBox.currentIndexChanged['QString'].connect(self.setTheme)
		self.statusBar.addPermanentWidget(self.themeBox)

		defaultTheme = "material"
		self.themeBox.setCurrentIndex(themes.index(defaultTheme))
		self.setTheme(defaultTheme)

		self.pushbutton.setMenu(menu)

		# self.userApplication = QtWidgets.QCheckBox("User App");
		# self.userApplication.toggled['bool'].connect(self.jumpToApplication)
		# self.statusBar.addPermanentWidget(self.userApplication)

		self.speedbutton = QtWidgets.QComboBox();
		self.speedbutton.addItems(['Slow', 'Fast', 'Ultra']);
		self.speedbutton.setCurrentIndex(1);
		self.speedbutton.currentIndexChanged['int'].connect(self.setSpeed)
		self.statusBar.addPermanentWidget(self.speedbutton)

		# Compile thread
		self.compile_thread_button = QtWidgets.QPushButton('Online Compiler')
		self.compile_thread_button.setStyleSheet("height: 13px;padding:3px;color: #7c7;")
		self.compile_thread_button.clicked.connect(self.activateCompileServer)
		self.statusBar.addPermanentWidget(self.compile_thread_button)

		self.bottomLabel = QtWidgets.QLabel("Messages")

		# Menu button
		self.statusBar.addPermanentWidget(self.pushbutton)

	def showServerStatus(self, msg):
		self.showStatus("Compiler: Error Launching Server (Restart app) ", True)
		QtWidgets.QMessageBox.warning(self, 'Server Error', msg)
		#self.statusBar.addPermanentWidget(self.compile_thread_button)

	def showStatus(self, msg, error=None):
		if error:
			self.statusBar.setStyleSheet("color:#F77")
		else:
			self.statusBar.setStyleSheet("color:#FFF")
		self.statusBar.showMessage(msg)

	def setSpeed(self, index):
		self.timer.setInterval([100, 20, 5][index])

	def locateDevices(self):
		try:
			L = KuttyPyLib.getFreePorts(self.p.portname)
		except Exception as e:
			print(e)
		total = len(L)
		menuChanged = False
		if L != self.shortlist:
			menuChanged = True
			if self.p.connected:
				if self.p.portname not in L:
					self.setWindowTitle('Error : Device Disconnected')
					QtWidgets.QMessageBox.warning(self, 'Connection Error',
					                              'Device Disconnected. Please check the connections')
					try:
						self.p.close()
					except:
						pass
					self.p.connected = False
					self.setWindowTitle('KuttyPy Interactive Console [ Hardware not detected ]')

			elif True in L.values():
				reply = QtWidgets.QMessageBox.question(self, 'Connection', 'Device Available. Connect?',
				                                       QtWidgets.QMessageBox.Yes, QtWidgets.QMessageBox.No)
				if reply == QtWidgets.QMessageBox.Yes:
					self.initializeCommunications()

			# update the shortlist
			self.shortlist = L

	def checkConnectionStatus(self, dialog=False):
		if self.p.connected:
			return True
		else:
			if dialog: QtWidgets.QMessageBox.warning(self, 'Connection Error',
			                                         'Device not connected. Please connect a KuttyPy to the USB port')
			return False

	######## WINDOW EXPORT SVG
	def exportSvg(self):
		from PyQt5 import QtSvg
		path, _filter = QtWidgets.QFileDialog.getSaveFileName(self, 'Save File', '~/')
		if path:
			generator = QtSvg.QSvgGenerator()
			generator.setFileName(path)
			target_rect = QtCore.QRectF(0, 0, 800, 600)
			generator.setSize(target_rect.size().toSize())  # self.size())
			generator.setViewBox(self.rect())
			generator.setTitle("Your title")
			generator.setDescription("some description")
			p = QtGui.QPainter()
			p.begin(generator)
			self.render(p)
			p.end()

	def ipython(self):  # Experimental feature. Import ipython and launch console
		if not self.p.connected:
			return
		from utilities import ipy
		if not self.ipy:
			self.ipy = ipy.AppWindow(self, kp=self.p)
		self.ipy.show()
		self.ipy.updateDevice(self.p)


def translators(langDir, lang=None):
	"""
    create a list of translators
    @param langDir a path containing .qm translation
    @param lang the preferred locale, like en_IN.UTF-8, fr_FR.UTF-8, etc.
    @result a list of QtCore.QTranslator instances
    """
	if lang == None:
		lang = QtCore.QLocale.system().name()
	result = []
	qtTranslator = QtCore.QTranslator()
	qtTranslator.load("qt_" + lang,
	                  QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.TranslationsPath))
	result.append(qtTranslator)

	# path to the translation files (.qm files)
	sparkTranslator = QtCore.QTranslator()
	sparkTranslator.load(lang, langDir);
	result.append(sparkTranslator)
	return result


def firstExistingPath(l):
	"""
    Returns the first existing path taken from a list of
    possible paths.
    @param l a list of paths
    @return the first path which exists in the filesystem, or None
    """
	for p in l:
		if os.path.exists(p):
			return p
	return None


def common_paths():
	"""
    Finds common paths
    @result a dictionary of common paths
    """
	path = {}
	curPath = os.path.dirname(os.path.realpath(__file__))
	path["current"] = curPath
	sharedPath = "/usr/share/kuttypy"
	path["translation"] = firstExistingPath(
		[os.path.join(p, "lang") for p in
		 (curPath, sharedPath,)])
	path["utilities"] = firstExistingPath(
		[os.path.join(p, 'utilities') for p in
		 (curPath, sharedPath,)])
	path["templates"] = firstExistingPath(
		[os.path.join(p, 'utilities', 'templates') for p in
		 (curPath, sharedPath,)])

	path["themes"] = firstExistingPath(
		[os.path.join(p, 'utilities', 'themes') for p in
		 (curPath, sharedPath,)])

	path["examples"] = firstExistingPath(
		[os.path.join(p, 'examples') for p in
		 (curPath, sharedPath,)])

	path["kpy"] = firstExistingPath(
		[os.path.join(p, 'kpy') for p in
		 (curPath, sharedPath,)])

	path["editor"] = firstExistingPath(
		[os.path.join(p, 'editor') for p in
		 (curPath, sharedPath,)])

	lang = str(QtCore.QLocale.system().name())
	shortLang = lang[:2]
	return path


def run():
	global path, app, myapp
	path = common_paths()
	print('QT Version', QtWidgets.__file__)
	app = QtWidgets.QApplication(sys.argv)
	myapp = AppWindow(app=app, path=path)
	myapp.show()
	r = app.exec_()
	if myapp.compile_thread is not None:
		myapp.compile_thread.terminate()
		print('waiting to quit compile_thread')
		myapp.compile_thread.wait()
	print('delete app..')
	app.deleteLater()
	sys.exit(r)


if __name__ == "__main__":
	run()



# Read MAX6675 temperature Values and plot them
import time
#from kuttyPy import *
from matplotlib import pyplot as plt
import numpy as np

#Outputs, pin definitions
PIN_SCK = 7; PIN_MOSI = 5 ; PIN_SS = 4
#$0D ($2D) SPCR : SPIE SPE DORD MSTR CPOL CPHA SPR1 SPR0 
SPE = 6 ; MSTR = 4 ; SPR0 = 0
PORTBvalue = 0
#Initialize things
setReg('DDRB', (1 << PIN_SCK) | (1 << PIN_MOSI) | (1 << PIN_SS)) #SCK MOSI CS/LOAD/SS
setReg('SPCR', (1 << SPE) | (1 << MSTR)| (1<<SPR0) ) #SPI Master Mode.

plt.ion() #Interactive plot
x=[] ; y = []
start_time = time.time() #Staring time noted
for a in range(5): #Get 50 readings 
	t = time.time() - start_time #Elapsed time

	PORTBvalue &= ~(1<<PIN_SS) # CS Low
	setReg('PORTB', PORTBvalue)

	setReg('SPDR', 0xFF)
	time.sleep(0.001)#Wait until transfer is complete . //while (!(SPSR & (1 << SPIF)));
	v1 = getReg('SPDR')

	setReg('SPDR', 0xFF)
	time.sleep(0.001)#Wait until transfer is complete . //while (!(SPSR & (1 << SPIF)));
	v2 = getReg('SPDR')

	PORTBvalue |= (1<<PIN_SS) #CS High
	setReg('PORTB', PORTBvalue)
	temp = (v1<<8)|v2
	if temp&0x4: #Invalid reading. thermocouple disconnected. Continue the loop from the top.
		continue
	temp = (temp >>3 )*0.25 #Right shift and discard first 3 bits. Divide by 4 to get Celcius equivalent


	print(t,temp)
	plt.scatter(t, temp , s=5) #Scatter plot (x= point number, y = temperature)
	x.append(t); y.append(temp)
	plt.pause(0.2) #Wait 200 mS 
print('press any key/button to close')
np.savetxt('tmp.txt',np.column_stack([x,y]),fmt='%1.2f')
plt.waitforbuttonpress()



## 'Qt' is a local module; it is intended mainly to cover up the differences
## between PyQt4 and PyQt5.
import os

ENVIRON = os.environ['CSMCA_QT_LIB']

if ENVIRON=='PyQt5':
	print('PyQt5 imported')
	from PyQt5 import QtGui,QtCore,QtWidgets
	try: from PyQt5 import QtSvg
	except: pass
	try:
		#from PyQt5.QtWebEngineWidgets import QWebEngineView as QWebView
		from PyQt5.QtWebKitWidgets import QWebView# , QWebPage
	except:
		QWebView = None
elif ENVIRON == 'PyQt4':
	print('PyQt4 imported')
	from PyQt4 import QtGui,QtCore
	from PyQt4 import QtGui as QtWidgets
	#try:
	#	from PyQt4.QtWebKitWidgets import QWebView# , QWebPage
	#except:
	#	print('webview unavailable')
	



from flask import Flask, request
from flask_cors import CORS
from PyQt5.QtCore import QThread, pyqtSignal
from PyQt5.QtWidgets import QApplication

# blueprint for socket comms parts of app
from .compile_routes import main as main_blueprint
from .compile_routes import setStatusSignal, setKpyPath
from werkzeug.serving import make_server
print('starting the compile server...')

flask_thread = None


def create_server(showStatusSignal, serverSignal, kpy_path, local_ip):
	setStatusSignal(showStatusSignal)
	setKpyPath(kpy_path, local_ip)
	flask_thread = FlaskThread()
	flask_thread.setServerSignal(serverSignal)
	# flask_thread.finished.connect(QApplication.quit)

	# Start the thread
	flask_thread.start()
	return flask_thread


class FlaskThread(QThread):
	finished = pyqtSignal()
	serverSignal = None

	def setServerSignal(self, sig):
		self.serverSignal = sig

	def run(self):
		# Run the Flask app in a separate thread
		print('starting the flask app...')
		self.app = Flask(__name__, template_folder='compile_templates')
		CORS(self.app)
		self.app.register_blueprint(main_blueprint)
		try:
			#self.app.run(host='0.0.0.0', port=5000)
			self.server = make_server('0.0.0.0', 5000, self.app)
			self.server.serve_forever()
		except Exception as e:
			import traceback
			self.serverSignal.emit(traceback.format_exc())

	def stop_flask_app(self):
		if hasattr(self, 'server') and self.server:
			self.server.shutdown()
			# Perform any necessary cleanup before stopping the app
			# Stop the Flask app


# -*- coding: utf-8; mode: python; indent-tabs-mode: t; tab-width:4 -*-
print('importing webbrowser')
import os,string,glob

from PyQt5 import QtGui,QtCore,QtWidgets
from PyQt5.QtWebEngineWidgets import QWebEngineView, QWebEnginePage
from PyQt5 import QtWebEngineWidgets



import sys,pkg_resources

class codeBrowser(QWebView):
	def __init__(self,*args,**kwargs):
		super(codeBrowser, self).__init__()
		self.code_path = '.'
		sys.path.append(self.code_path)
		self.showMaximized()
				
	def setFile(self,url):
		newUrl = QtCore.QUrl.fromLocalFile(QtCore.QFileInfo(url).absoluteFilePath())
		print ('SETTING URL',url,newUrl)
		self.setUrl(newUrl)#pkg_resources.resource_filename('eyes_html',url)))



from PyQt5.QtGui import QKeySequence
from PyQt5.QtWidgets import QTabWidget, QShortcut

from utilities import Qt


class MyTabBrowser(QTabWidget):
    def __init__(self):
        super().__init__()
        # Set up shortcuts for Ctrl+Left and Ctrl+Right
        self.prev_tab_shortcut = QShortcut(QKeySequence(Qt.CTRL + Qt.Key_Left), self)
        self.next_tab_shortcut = QShortcut(QKeySequence(Qt.CTRL + Qt.Key_Right), self)

        # Connect shortcuts to navigation functions
        self.prev_tab_shortcut.activated.connect(self.showPreviousTab)
        self.next_tab_shortcut.activated.connect(self.showNextTab)

    def keyPressEvent(self, event):
        # Override the keyPressEvent to prevent default handling of Ctrl+Tab
        if event.key() == Qt.Key_Tab and event.modifiers() == Qt.ControlModifier:
            return

        # Call the base class implementation for other key events
        super().keyPressEvent(event)

    def showPreviousTab(self):
        current_index = self.currentIndex()
        if current_index > 0:
            self.setCurrentIndex(current_index - 1)

    def showNextTab(self):
        current_index = self.currentIndex()
        if current_index < self.count() - 1:
            self.setCurrentIndex(current_index + 1)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'layout_ide.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(985, 565)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/control/kuttypy.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        MainWindow.setWindowIcon(icon)
        MainWindow.setStyleSheet("")
        MainWindow.setDockOptions(QtWidgets.QMainWindow.AnimatedDocks)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_2.setSizeConstraint(QtWidgets.QLayout.SetMaximumSize)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setSpacing(0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setContentsMargins(-1, -1, -1, 0)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.logLabel = QtWidgets.QLabel(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.logLabel.sizePolicy().hasHeightForWidth())
        self.logLabel.setSizePolicy(sizePolicy)
        self.logLabel.setMinimumSize(QtCore.QSize(0, 30))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setItalic(True)
        self.logLabel.setFont(font)
        self.logLabel.setAlignment(QtCore.Qt.AlignBottom|QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft)
        self.logLabel.setObjectName("logLabel")
        self.horizontalLayout.addWidget(self.logLabel)
        self.gridLayout_2.addLayout(self.horizontalLayout, 0, 1, 1, 1)
        self.frame = QtWidgets.QFrame(self.centralwidget)
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.frame)
        self.gridLayout_6.setContentsMargins(2, 2, 2, 2)
        self.gridLayout_6.setSpacing(2)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.frame_2 = QtWidgets.QFrame(self.frame)
        self.frame_2.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_2.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_2.setObjectName("frame_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.frame_2)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setSpacing(3)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.filenameLabel = QtWidgets.QLabel(self.frame_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.filenameLabel.sizePolicy().hasHeightForWidth())
        self.filenameLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(9)
        self.filenameLabel.setFont(font)
        self.filenameLabel.setObjectName("filenameLabel")
        self.horizontalLayout_2.addWidget(self.filenameLabel)
        self.fontComboBox = QtWidgets.QFontComboBox(self.frame_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.fontComboBox.sizePolicy().hasHeightForWidth())
        self.fontComboBox.setSizePolicy(sizePolicy)
        self.fontComboBox.setObjectName("fontComboBox")
        self.horizontalLayout_2.addWidget(self.fontComboBox)
        self.pushButton_3 = QtWidgets.QPushButton(self.frame_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButton_3.sizePolicy().hasHeightForWidth())
        self.pushButton_3.setSizePolicy(sizePolicy)
        self.pushButton_3.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/control/plus.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_3.setIcon(icon1)
        self.pushButton_3.setObjectName("pushButton_3")
        self.horizontalLayout_2.addWidget(self.pushButton_3)
        self.pushButton_4 = QtWidgets.QPushButton(self.frame_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButton_4.sizePolicy().hasHeightForWidth())
        self.pushButton_4.setSizePolicy(sizePolicy)
        self.pushButton_4.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/control/minus.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_4.setIcon(icon2)
        self.pushButton_4.setObjectName("pushButton_4")
        self.horizontalLayout_2.addWidget(self.pushButton_4)
        self.gridLayout_6.addWidget(self.frame_2, 3, 0, 1, 1)
        self.codingTabs = QtWidgets.QTabWidget(self.frame)
        self.codingTabs.setTabsClosable(True)
        self.codingTabs.setMovable(True)
        self.codingTabs.setObjectName("codingTabs")
        self.gridLayout_6.addWidget(self.codingTabs, 2, 0, 1, 1)
        self.frame_4 = QtWidgets.QFrame(self.frame)
        self.frame_4.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_4.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_4.setObjectName("frame_4")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.frame_4)
        self.horizontalLayout_5.setContentsMargins(2, 2, 2, 2)
        self.horizontalLayout_5.setSpacing(2)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.fileMenuButton = QtWidgets.QPushButton(self.frame_4)
        self.fileMenuButton.setObjectName("fileMenuButton")
        self.horizontalLayout_5.addWidget(self.fileMenuButton)
        self.editMenuButton = QtWidgets.QPushButton(self.frame_4)
        self.editMenuButton.setObjectName("editMenuButton")
        self.horizontalLayout_5.addWidget(self.editMenuButton)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem)
        self.compileButton = QtWidgets.QPushButton(self.frame_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.compileButton.sizePolicy().hasHeightForWidth())
        self.compileButton.setSizePolicy(sizePolicy)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/control/refresh.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.compileButton.setIcon(icon3)
        self.compileButton.setObjectName("compileButton")
        self.horizontalLayout_5.addWidget(self.compileButton)
        self.buildOptionsButton = QtWidgets.QPushButton(self.frame_4)
        self.buildOptionsButton.setText("")
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/control/settings.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.buildOptionsButton.setIcon(icon4)
        self.buildOptionsButton.setObjectName("buildOptionsButton")
        self.horizontalLayout_5.addWidget(self.buildOptionsButton)
        self.uploadButton = QtWidgets.QPushButton(self.frame_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uploadButton.sizePolicy().hasHeightForWidth())
        self.uploadButton.setSizePolicy(sizePolicy)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/control/download.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.uploadButton.setIcon(icon5)
        self.uploadButton.setObjectName("uploadButton")
        self.horizontalLayout_5.addWidget(self.uploadButton)
        self.launchFirmwareButton = QtWidgets.QCheckBox(self.frame_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.launchFirmwareButton.sizePolicy().hasHeightForWidth())
        self.launchFirmwareButton.setSizePolicy(sizePolicy)
        self.launchFirmwareButton.setObjectName("launchFirmwareButton")
        self.horizontalLayout_5.addWidget(self.launchFirmwareButton)
        self.gridLayout_6.addWidget(self.frame_4, 0, 0, 1, 1)
        self.reloadFrame = QtWidgets.QFrame(self.frame)
        self.reloadFrame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.reloadFrame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.reloadFrame.setObjectName("reloadFrame")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.reloadFrame)
        self.horizontalLayout_3.setContentsMargins(2, 2, 2, 2)
        self.horizontalLayout_3.setSpacing(2)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_2 = QtWidgets.QLabel(self.reloadFrame)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_3.addWidget(self.label_2)
        self.reloadLabel = QtWidgets.QLabel(self.reloadFrame)
        self.reloadLabel.setObjectName("reloadLabel")
        self.horizontalLayout_3.addWidget(self.reloadLabel)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.pushButton = QtWidgets.QPushButton(self.reloadFrame)
        self.pushButton.setObjectName("pushButton")
        self.horizontalLayout_3.addWidget(self.pushButton)
        self.pushButton_2 = QtWidgets.QPushButton(self.reloadFrame)
        self.pushButton_2.setText("")
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/control/close.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_2.setIcon(icon6)
        self.pushButton_2.setObjectName("pushButton_2")
        self.horizontalLayout_3.addWidget(self.pushButton_2)
        self.gridLayout_6.addWidget(self.reloadFrame, 1, 0, 1, 1)
        self.gridLayout_2.addWidget(self.frame, 0, 0, 5, 1)
        self.termFrame = QtWidgets.QFrame(self.centralwidget)
        self.termFrame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.termFrame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.termFrame.setObjectName("termFrame")
        self.termLayout = QtWidgets.QVBoxLayout(self.termFrame)
        self.termLayout.setContentsMargins(0, 0, 0, 0)
        self.termLayout.setSpacing(0)
        self.termLayout.setObjectName("termLayout")
        self.gridLayout_2.addWidget(self.termFrame, 4, 1, 1, 1)
        self.log = QtWidgets.QTextBrowser(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.log.sizePolicy().hasHeightForWidth())
        self.log.setSizePolicy(sizePolicy)
        self.log.setMinimumSize(QtCore.QSize(400, 0))
        font = QtGui.QFont()
        font.setPointSize(11)
        self.log.setFont(font)
        self.log.setLineWrapMode(QtWidgets.QTextEdit.WidgetWidth)
        self.log.setObjectName("log")
        self.gridLayout_2.addWidget(self.log, 3, 1, 1, 1)
        self.serialFrame = QtWidgets.QFrame(self.centralwidget)
        self.serialFrame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.serialFrame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.serialFrame.setObjectName("serialFrame")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.serialFrame)
        self.horizontalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_4.setSpacing(2)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.serialData = QtWidgets.QLineEdit(self.serialFrame)
        self.serialData.setObjectName("serialData")
        self.horizontalLayout_4.addWidget(self.serialData)
        self.sendASCIIButton = QtWidgets.QPushButton(self.serialFrame)
        self.sendASCIIButton.setObjectName("sendASCIIButton")
        self.horizontalLayout_4.addWidget(self.sendASCIIButton)
        self.sendBinaryButton = QtWidgets.QPushButton(self.serialFrame)
        self.sendBinaryButton.setObjectName("sendBinaryButton")
        self.horizontalLayout_4.addWidget(self.sendBinaryButton)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem2)
        self.serialGuageButton = QtWidgets.QPushButton(self.serialFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.serialGuageButton.sizePolicy().hasHeightForWidth())
        self.serialGuageButton.setSizePolicy(sizePolicy)
        self.serialGuageButton.setMaximumSize(QtCore.QSize(90, 16777215))
        self.serialGuageButton.setObjectName("serialGuageButton")
        self.horizontalLayout_4.addWidget(self.serialGuageButton)
        self.gridLayout_2.addWidget(self.serialFrame, 1, 1, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        self.codingTabs.setCurrentIndex(-1)
        self.serialGuageButton.clicked.connect(MainWindow.showSerialGauge) # type: ignore
        self.compileButton.clicked.connect(MainWindow.compile) # type: ignore
        self.uploadButton.clicked.connect(MainWindow.upload) # type: ignore
        self.launchFirmwareButton.clicked['bool'].connect(MainWindow.jumpToApplication) # type: ignore
        self.pushButton_4.clicked.connect(MainWindow.fontMinus) # type: ignore
        self.pushButton_3.clicked.connect(MainWindow.fontPlus) # type: ignore
        self.fontComboBox.currentTextChanged['QString'].connect(MainWindow.setFont) # type: ignore
        self.pushButton.clicked.connect(MainWindow.reloadFile) # type: ignore
        self.pushButton_2.clicked.connect(MainWindow.cancelReload) # type: ignore
        self.sendBinaryButton.clicked.connect(MainWindow.sendBinary) # type: ignore
        self.sendASCIIButton.clicked.connect(MainWindow.sendASCII) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "KuttyPy Interactive Console"))
        self.logLabel.setText(_translate("MainWindow", "Monitor registers being read and set during each operation"))
        self.filenameLabel.setText(_translate("MainWindow", ":"))
        self.fileMenuButton.setText(_translate("MainWindow", "File"))
        self.editMenuButton.setText(_translate("MainWindow", "Edit"))
        self.compileButton.setText(_translate("MainWindow", "Compile/Assemble"))
        self.uploadButton.setText(_translate("MainWindow", "Upload"))
        self.launchFirmwareButton.setText(_translate("MainWindow", "Run"))
        self.label_2.setText(_translate("MainWindow", "File Changed:"))
        self.reloadLabel.setText(_translate("MainWindow", ".."))
        self.pushButton.setText(_translate("MainWindow", "RELOAD"))
        self.sendASCIIButton.setText(_translate("MainWindow", "Send"))
        self.sendBinaryButton.setText(_translate("MainWindow", "Send Binary"))
        self.serialGuageButton.setToolTip(_translate("MainWindow", "Show serial port data in the form of a plot or dial"))
        self.serialGuageButton.setText(_translate("MainWindow", "Gauge"))
from . import resplus_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'dummyio.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(291, 25)
        Form.setMaximumSize(QtCore.QSize(16777215, 25))
        Form.setStyleSheet("border-color: rgb(3, 55, 58);")
        self.horizontalLayout = QtWidgets.QHBoxLayout(Form)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setSpacing(2)
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(30, 25, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.nameIn = QtWidgets.QPushButton(Form)
        self.nameIn.setMaximumSize(QtCore.QSize(16777215, 20))
        self.nameIn.setObjectName("nameIn")
        self.horizontalLayout.addWidget(self.nameIn)
        spacerItem1 = QtWidgets.QSpacerItem(30, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.nameIn.setText(_translate("Form", "-"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'dio_stepper.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(627, 326)
        self.gridLayout = QtWidgets.QGridLayout(Dialog)
        self.gridLayout.setContentsMargins(5, 5, 5, 5)
        self.gridLayout.setSpacing(3)
        self.gridLayout.setObjectName("gridLayout")
        self.pushButton_5 = QtWidgets.QPushButton(Dialog)
        self.pushButton_5.setAutoRepeat(True)
        self.pushButton_5.setAutoRepeatInterval(20)
        self.pushButton_5.setAutoDefault(False)
        self.pushButton_5.setObjectName("pushButton_5")
        self.gridLayout.addWidget(self.pushButton_5, 0, 3, 1, 1)
        self.pushButton_2 = QtWidgets.QPushButton(Dialog)
        font = QtGui.QFont()
        font.setPointSize(13)
        self.pushButton_2.setFont(font)
        self.pushButton_2.setAutoRepeat(True)
        self.pushButton_2.setAutoRepeatInterval(20)
        self.pushButton_2.setAutoDefault(False)
        self.pushButton_2.setObjectName("pushButton_2")
        self.gridLayout.addWidget(self.pushButton_2, 0, 2, 1, 1)
        self.pushButton = QtWidgets.QPushButton(Dialog)
        self.pushButton.setAutoDefault(False)
        self.pushButton.setObjectName("pushButton")
        self.gridLayout.addWidget(self.pushButton, 0, 0, 1, 1)
        self.monitors = QtWidgets.QStackedWidget(Dialog)
        self.monitors.setObjectName("monitors")
        self.page_1 = QtWidgets.QWidget()
        self.page_1.setObjectName("page_1")
        self.gaugeLayout = QtWidgets.QHBoxLayout(self.page_1)
        self.gaugeLayout.setContentsMargins(5, 5, 5, 5)
        self.gaugeLayout.setObjectName("gaugeLayout")
        self.monitors.addWidget(self.page_1)
        self.page_2 = QtWidgets.QWidget()
        self.page_2.setObjectName("page_2")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.page_2)
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_3.setSpacing(0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.graph = PlotWidget(self.page_2)
        self.graph.setObjectName("graph")
        self.horizontalLayout_3.addWidget(self.graph)
        self.monitors.addWidget(self.page_2)
        self.gridLayout.addWidget(self.monitors, 3, 0, 1, 6)
        self.currentPositionBox = QtWidgets.QSpinBox(Dialog)
        self.currentPositionBox.setMinimum(-10000)
        self.currentPositionBox.setMaximum(10000)
        self.currentPositionBox.setObjectName("currentPositionBox")
        self.gridLayout.addWidget(self.currentPositionBox, 0, 4, 1, 1)
        self.frame = QtWidgets.QFrame(Dialog)
        self.frame.setMaximumSize(QtCore.QSize(80, 16777215))
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.frame)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setSpacing(3)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.pushButton_3 = QtWidgets.QPushButton(self.frame)
        self.pushButton_3.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/control/plus.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_3.setIcon(icon)
        self.pushButton_3.setAutoDefault(False)
        self.pushButton_3.setObjectName("pushButton_3")
        self.verticalLayout_3.addWidget(self.pushButton_3)
        self.listWidget = QtWidgets.QListWidget(self.frame)
        self.listWidget.setObjectName("listWidget")
        self.verticalLayout_3.addWidget(self.listWidget)
        self.pushButton_4 = QtWidgets.QPushButton(self.frame)
        self.pushButton_4.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/control/play.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_4.setIcon(icon1)
        self.pushButton_4.setAutoDefault(False)
        self.pushButton_4.setObjectName("pushButton_4")
        self.verticalLayout_3.addWidget(self.pushButton_4)
        self.gridLayout.addWidget(self.frame, 0, 6, 6, 1)
        self.spinBox_2 = QtWidgets.QSpinBox(Dialog)
        self.spinBox_2.setObjectName("spinBox_2")
        self.gridLayout.addWidget(self.spinBox_2, 4, 3, 1, 1)
        self.pinSet = QtWidgets.QComboBox(Dialog)
        self.pinSet.setFrame(True)
        self.pinSet.setObjectName("pinSet")
        self.pinSet.addItem("")
        self.pinSet.addItem("")
        self.gridLayout.addWidget(self.pinSet, 4, 0, 1, 3)
        self.label = QtWidgets.QLabel(Dialog)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 4, 4, 1, 2)
        self.configLayout = QtWidgets.QHBoxLayout()
        self.configLayout.setObjectName("configLayout")
        self.gridLayout.addLayout(self.configLayout, 2, 0, 1, 6)

        self.retranslateUi(Dialog)
        self.monitors.setCurrentIndex(0)
        self.pushButton_3.clicked.connect(Dialog.add) # type: ignore
        self.pushButton_4.clicked.connect(Dialog.play) # type: ignore
        self.pushButton.clicked.connect(Dialog.initialize) # type: ignore
        self.pushButton_2.clicked.connect(Dialog.stepLeft) # type: ignore
        self.pushButton_5.clicked.connect(Dialog.stepRight) # type: ignore
        self.currentPositionBox.editingFinished.connect(Dialog.stepTo) # type: ignore
        self.pinSet.currentIndexChanged['int'].connect(Dialog.setPins) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.pushButton_5.setText(_translate("Dialog", ">"))
        self.pushButton_2.setText(_translate("Dialog", "<"))
        self.pushButton.setText(_translate("Dialog", "HOME"))
        self.pinSet.setItemText(0, _translate("Dialog", "A+ : PB0, B + PB1, A- : PB2, B- : PB3"))
        self.pinSet.setItemText(1, _translate("Dialog", "PB4,PB5,PB6,PB7"))
        self.label.setText(_translate("Dialog", "Steps per full revolution"))
from pyqtgraph import PlotWidget
from . import res_rc



import os

from .Qt import QtGui,QtCore,QtWidgets
from .templates import ui_ipy as ipy_template
import sys,string
import time

from qtconsole.rich_jupyter_widget import RichJupyterWidget
from qtconsole.inprocess import QtInProcessKernelManager

# The ID of an installed kernel, e.g. 'bash' or 'ir'.
USE_KERNEL = 'python3'

class myConsole(RichJupyterWidget):
	def __init__(self,customBanner=None):
		"""Start a kernel, connect to it, and create a RichJupyterWidget to use it
		"""
		super(myConsole, self).__init__()
		if customBanner!=None: self.banner=customBanner
		self.kernel_manager = QtInProcessKernelManager(kernel_name=USE_KERNEL)
		self.kernel_manager.start_kernel()
		self.kernel = self.kernel_manager.kernel

		self.kernel_manager.kernel.gui = 'qt'
		self.font_size = 6

		self.kernel_client = self.kernel_manager.client()
		self.kernel_client.start_channels()

		def stop():
			kernel_client.stop_channels()
			kernel_manager.shutdown_kernel()
			guisupport.get_app_qt().exit()

		self.exit_requested.connect(stop)

	def pushVariables(self,variableDict):
		""" Given a dictionary containing name / value pairs, push those variables to the IPython console widget """
		self.kernel.shell.push(variableDict)
	def clearTerminal(self):
		""" Clears the terminal """
		self._control.clear()    

	def printText(self,text):
		""" Prints some plain text to the console """
		self._append_plain_text(text)

	def executeCommand(self,command,hidden=False):
		""" Execute a command in the frame of the console widget """
		self._execute(command,hidden)



class AppWindow(QtWidgets.QMainWindow, ipy_template.Ui_MainWindow):
	def __init__(self, parent=None,**kwargs):
		super(AppWindow, self).__init__(parent)
		self.setupUi(self)
		self.kp=kwargs.get('kp',None)

		self.setWindowTitle('iPython Console : Try out stuff!')
		self.msg = QtWidgets.QLabel()
		self.statusbar.addWidget(self.msg)
		self.msg.setText('Hi!')
		self.timer = QtCore.QTimer()

		try:
			#--------instantiate the iPython class-------
			self.ipyConsole = myConsole("################ Interactive KuttyPy Console ###########\nAccess hardware using the Instance 'kp'.  e.g.  kp.setReg('PORTB',8)\n\n")
			self.layout.addWidget(self.ipyConsole)
		except Exception as e:
			print ("failed to launch iPython. Is it installed?", e)
			self.close()
			return
			

		cmdDict = {}
		#cmdDict = {"analytics":self.analytics}
		if self.kp :
			cmdDict["kp"]=self.kp
		self.ipyConsole.pushVariables(cmdDict)
		self.console_enabled=True

	def importNumpy(self):
		self.ipyConsole.executeCommand('import numpy as np',True)
		self.message('imported Numpy as np')

	def importScipy(self):
		self.ipyConsole.executeCommand('import scipy',True)
		self.message('imported scipy')

	def importPylab(self):
		self.msg.setText('importing Pylab...')
		self.ipyConsole.executeCommand('from pylab import *',True)
		self.message('from pylab import * .  You can use plot() and show() commands now.')

	def message(self,txt):
		self.msg.setText(txt)
		self.timer.stop()
		self.timer.singleShot(4000,self.msg.clear)

	def updateDevice(self,kp):
		self.kp = kp

	def closeEvent(self, event):
		self.timer.stop()
		self.finished=True

	def __del__(self):
		print ('bye')
        		
if __name__ == "__main__":
	app = QtWidgets.QApplication(sys.argv)
	myapp = AppWindow(kp=None)
	myapp.show()
	sys.exit(app.exec_())





#!/usr/bin/python3
'''
A record (line of text) consists of six fields (parts) that appear in order from left to right:

Start code, one character, an ASCII colon ':'.
Byte count, two hex digits, indicating the number of bytes (hex digit pairs) in the data field. The maximum byte count is 255 (0xFF). 16 (0x10) and 32 (0x20) are commonly used byte counts.
Address, four hex digits, representing the 16-bit beginning memory address offset of the data. The physical address of the data is computed by adding this offset to a previously established base address, thus allowing memory addressing beyond the 64 kilobyte limit of 16-bit addresses. The base address, which defaults to zero, can be changed by various types of records. Base addresses and address offsets are always expressed as big endian values.
Record type (see record types below), two hex digits, 00 to 05, defining the meaning of the data field.
Data, a sequence of n bytes of data, represented by 2n hex digits. Some records omit this field (n equals zero). The meaning and interpretation of data bytes depends on the application.
Checksum, two hex digits, a computed value that can be used to verify the record has no errors.
'''
import time
import serial,struct
Byte =     struct.Struct("B") # size 1


class Uploader(object):
	STK_OK = 0x10
	STK_INSYNC = 0x14  # ' '
	CRC_EOP = 0x20  # 'SPACE'
	STK_GET_SYNC = 0x30  # '0'
	STK_SET_PARAMETER = 0x40  # '@'
	STK_GET_PARAMETER = 0x41  # 'A'
	STK_SET_DEVICE = 0x42  # 'B'
	FLASH_MEMORY = 0x46  #'F'
	STK_ENTER_PROGMODE = 0x50  # 'P'
	STK_LEAVE_PROGMODE = 0x51  # 'Q'
	STK_CHIP_ERASE = 0x52  # 'R'
	STK_LOAD_ADDRESS = 0x55  # 'U'
	STK_PROG_FLASH = 0x60  # '`'
	STK_PROG_DATA = 0x61  # 'a'
	STK_PROG_FUSE = 0x62  # 'b'
	STK_PROG_LOCK = 0x63  # 'c'
	STK_PROG_PAGE = 0x64  # 'd'
	STK_PROG_FUSE_EXT = 0x65  # 'e'
	STK_READ_FLASH = 0x70  # 'p'
	STK_READ_DATA = 0x71  # 'q'
	STK_READ_FUSE = 0x72  # 'r'
	STK_READ_LOCK = 0x73  # 's'
	STK_READ_PAGE = 0x74  # 't'
	STK_READ_SIGN = 0x75  # 'u'
	STK_READ_OSCCAL = 0x76  # 'v'
	STK_READ_FUSE_EXT = 0x77  # 'w'
	STK_READ_OSCCAL_EXT = 0x78  # 'x'


	SYNC = [STK_GET_SYNC, CRC_EOP]
	CHIP_ERASE = [STK_CHIP_ERASE, CRC_EOP]
	ENTER_PROG_MODE = [STK_ENTER_PROGMODE, CRC_EOP]
	EXIT_PROG_MODE = [STK_LEAVE_PROGMODE, CRC_EOP]
	INSYNC = [STK_INSYNC, STK_OK]
	def __init__(self, sock, retry=3, hexfile="",logger=None):
		self.sock = sock
		self.hexfile = hexfile
		self.retry = retry
		if logger:
			self.log = logger.emit
		else:
			self.log = print

	def spi_transaction(self, codes, bytesreply=0):
		n = 0
		retry = True
		tx_complete = len(codes) * 0.25 + 1
		while retry:
			self.sock.write(codes)
			time.sleep(tx_complete/1000.0)
			reply = list(self.sock.read(size=bytesreply + 2)) #bytesreply + INSYNC + OK
			print('reply', len(reply), reply)
			if not reply or ([reply[0], reply[-1]] != self.INSYNC):
				if n < self.retry:
					n += 1
					self.log('retrying...%s'%reply)
					continue
				else:
					raise Exception("SPI","Not in sync")
					return
			if len(reply) == 3:
				return reply[1]
			elif len(reply) > 3:
				return reply[1:-1]
			return
	
	def sync(self):
		self.sock.write(b'0 ')#\x30\x20')
		x = self.sock.read(2)
		print('sync',x)

	
	def program(self):
		st = time.time()
		print('start programming. flush:',self.sock.read(1))
		#self.sync()
		print('synced',(time.time()-st)*1000)
		# print("erasing...")
		# self.spi_transaction(CHIP_ERASE)

		self.log("Entering programming mode")
		self.spi_transaction(self.ENTER_PROG_MODE)

		# start with page address 0
		address = 0
		prg_length = 0
		data = list()
		print('entered program mode',(time.time()-st)*1000)

		# open the hex file
		with open(self.hexfile, "rb") as hexfile:
			while True:
				row = hexfile.readline()
				# check EOF
				# Include only program data
				if row[7:9] != b'01':
					hexrow = row[9:][:-4]
					data.extend([int(hexrow[b:b + 2], 16) for b in range(len(hexrow))[::2]])
				if not data:
					self.log("End program")
					break
				if len(data) >= 128 or row[7:9] == b'01':
					size = len(data[:128])
					prg_length += size
					#print('writing', len(data), (time.time() - st) * 1000)

					self.spi_transaction([self.STK_LOAD_ADDRESS, address % 256, int(address / 256), self.CRC_EOP])
					self.log("Writing @ %s:%s, block size:%s"%( int(address / 256), address % 256, size ))
					address += 64
					packet = [self.STK_PROG_PAGE, 0, size, self.FLASH_MEMORY] + data[:128] + [self.CRC_EOP]
					self.spi_transaction(packet)
					#print ('writing', len(packet), packet)
					data = data[128:]

		self.spi_transaction(self.EXIT_PROG_MODE)
		self.sync()
		#print("Finished. Program size %s bytes in %.1f mS"%(prg_length,(time.time()-st)*1000))
		self.log("Finished. Program size %s bytes in %.1f mS"%(prg_length,(time.time()-st)*1000))

	def verify(self):
		print('entered verify mode')
		self.sync()
		self.spi_transaction(self.ENTER_PROG_MODE)
		address = 0
		with open(self.hexfile, "rb") as hexfile:
			while True:
				self.spi_transaction([self.STK_LOAD_ADDRESS, address % 256, int(address / 256), self.CRC_EOP])
	
				data = list()
				for i in range(8):
					hexrow = hexfile.readline()[9:][:-4]
					data.extend([int(hexrow[b:b + 2], 16) for b in range(len(hexrow))[::2]])
	
				size = len(data)
				self.log("Reading program page( 128 bytes / 64 words ) Starts at:%s:%s"%(int(address / 256), address % 256))
				page = self.spi_transaction([self.STK_READ_PAGE, 0, 0x80, self.FLASH_MEMORY, self.CRC_EOP], 0x80)
				if data != page[:size]:
					self.log("Error! Differs from Hex file")
					self.spi_transaction(self.EXIT_PROG_MODE)
					return False

				if size != 0x80:
					self.log("Program check OK.")
					self.spi_transaction(self.EXIT_PROG_MODE)
					return True
				address += 64


if __name__ == '__main__':
	ser = serial.Serial('/dev/ttyUSB0', baudrate=38400, timeout=.5)
	ser.setRTS(0)
	ser.setRTS(1)
	time.sleep(0.02)
	dude = Uploader(ser, hexfile="./blink.hex")
	dude.program()
	dude.verify()
	ser.setRTS(0)
	ser.setRTS(1)
	time.sleep(0.02)



'''
'''
#!/usr/bin/python3

import os,sys,time,re,traceback,platform,inspect
from utilities.Qt import QtGui, QtCore, QtWidgets
import KuttyPyLibUno

from utilities.templates import ui_layoutuno as layout
from utilities import dio,uploader
from utilities import REGISTERS_UNO as REGISTERS
import constants


from functools import partial
from collections import OrderedDict




class myTimer():
	def __init__(self,interval):
		self.interval = interval
		self.reset()
	def reset(self):
		self.timeout = time.time()+self.interval
	def ready(self):
		T = time.time()
		dt = T - self.timeout
		if dt>0: #timeout is ahead of current time 
			#if self.interval>5:print('reset',self.timeout,dt)
			self.timeout = T - dt%self.interval + self.interval
			#if self.interval>5:print(self.timeout)
			return True
		return False
	def progress(self):
		return 100*(self.interval - self.timeout + time.time())/(self.interval)			

class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
	p=None
	logThis = QtCore.pyqtSignal(str)
	logThisPlain = QtCore.pyqtSignal(bytes)
	serialGaugeSignal = QtCore.pyqtSignal(int)
	def __init__(self, parent=None,**kwargs):
		super(AppWindow, self).__init__(parent)
		self.setupUi(self)

		self.VERSION = REGISTERS.VERSIONNUM
		self.SPECIALS = REGISTERS.SPECIALS
		self.REGISTERS = REGISTERS.REGISTERS
		self.EXAMPLES_DIR = REGISTERS.EXAMPLES
		self.ADC_PINS = REGISTERS.ADC
		
		self.docks = [self.leftdock,self.rightdock]
		self.sensorList = []
		self.controllerList = []
		self.monitoring = True
		self.logRegisters = True
		self.userHexRunning = False
		self.uploadingHex = False
		self.autoUpdateUserRegisters = False
		self.CFile = None #'~/kuttyPy.c'
		self.ipy = None

		self.setTheme("material")
		examples = [a for a in os.listdir(os.path.join(path["examples"],self.EXAMPLES_DIR)) if ('.py' in a) and a is not 'kuttyPy.py'] #.py files except the library
		self.exampleList.addItems(examples)
		blinkindex = self.exampleList.findText('blink.py')
		if blinkindex!=-1: #default example. blink.py present in examples directory
			self.exampleList.setCurrentIndex(blinkindex)

		######## PYTHON CODE
		self.codeThread = QtCore.QThread()
		self.codeEval = self.codeObject(self.REGISTERS)
		self.codeEval.moveToThread(self.codeThread)
		self.codeEval.finished.connect(self.codeThread.quit)
		self.codeEval.logThis.connect(self.appendLog) #Connect to the log window
		self.logThis.connect(self.appendLog) #Connect to the log window
		self.logThisPlain.connect(self.appendLogPlain) #Connect to the log window
		self.serialGaugeSignal.connect(self.setSerialgauge)
		
		self.codeThread.started.connect(self.codeEval.execute)
		self.codeThread.finished.connect(self.codeFinished)

		######### C CODE UPLOADER
		self.uploadThread = QtCore.QThread()
		self.UploadObject = self.uploadObject()
		self.UploadObject.moveToThread(self.uploadThread)
		self.UploadObject.finished.connect(self.uploadThread.quit)
		self.UploadObject.logThis.connect(self.appendLog) #Connect to the log window
		self.UploadObject.logThisPlain.connect(self.appendLogPlain) #Connect to the log window. add plain text
		self.logThis.connect(self.appendLog) #Connect to the log window

		self.uploadThread.started.connect(self.UploadObject.execute)
		self.uploadThread.finished.connect(self.codeFinished)

		self.commandQ = []
		self.btns={}
		self.registers = []
		self.addPins()

		self.statusBar = self.statusBar()
		self.makeBottomMenu()

		global app


		self.initializeCommunications()
		self.pending = {
		'status':myTimer(constants.STATUS_UPDATE_INTERVAL),
		'update':myTimer(constants.AUTOUPDATE_INTERVAL),
		}

		serialgaugeoptions = {'name':'Serial Monitor', 'init':print, 'read':None,
				'fields':['Value'],
				'min':[0],
				'max':[255]}
		self.serialGauge = dio.DIOSENSOR(self,serialgaugeoptions)
		
		self.startTime = time.time()
		self.timer = QtCore.QTimer()
		self.timer.timeout.connect(self.updateEverything)
		self.timer.start(20)

		
		#Auto-Detector
		self.shortlist=KuttyPyLibUno.getFreePorts()

	def newRegister(self):
		reg = dio.REGEDIT(self.commandQ)
		#self.registerLayout.addWidget(reg)
		#self.registers.append(reg)

		#TODO: Convert layout to listwidget to enable re-ordering
		regItem = QtWidgets.QListWidgetItem()
		regItem.setSizeHint(QtCore.QSize(200,40))
		self.registerList.addItem(regItem)
		self.registerList.setItemWidget(regItem,reg)
		self.registers.append(regItem)

	def addPins(self):
		'''
		This function adds pins in the right order. this should be moved to a configuration file!
		'''
		self.btns['C'] = dio.REGVALS('C')
		self.leftlayout.addWidget(self.btns['C'])
		s = QtWidgets.QSpacerItem(20,250,QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
		self.leftlayout.addItem(s)

		for name in ['PC0','PC1','PC2','PC3','PC4','PC5']: #left dock
			checkbox = dio.widget(name,self.commandQ,extra = self.SPECIALS.get(name,''))
			self.leftlayout.addWidget(checkbox)
			self.btns[name] = checkbox

		self.btns['B'] = dio.REGVALS('B')
		self.rightlayout.addWidget(self.btns['B'])

		for name in ['PB5','PB4','PB3','PB2','PB1','PB0']: #right dock
			checkbox = dio.widget(name,self.commandQ,extra = self.SPECIALS.get(name,''))
			self.rightlayout.addWidget(checkbox)
			self.btns[name] = checkbox

		s = QtWidgets.QSpacerItem(20,20,QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
		self.rightlayout.addItem(s)

		self.btns['D'] = dio.REGVALS('D')
		self.rightlayout.addWidget(self.btns['D'])
		for name in ['PD7','PD6','PD5','PD4','PD3','PD2','PD1','PD0']: #right dock
			checkbox = dio.widget(name,self.commandQ,extra = self.SPECIALS.get(name,''))
			self.rightlayout.addWidget(checkbox)
			self.btns[name] = checkbox

	def tabChanged(self,index):
		if index != 0 : #examples/editor tab. disable monitoring
			self.monitoring = False
			for a in self.docks:
				a.hide()		
		else: #Playground . enable monitoring and control.
			self.monitoring = True
			self.autoRefreshUserRegisters.setChecked(False)
			self.userRegistersAutoRefresh(False)
			for a in self.docks:
				a.show()		

	################USER CODE SECTION####################
	class codeObject(QtCore.QObject):
		finished = QtCore.pyqtSignal()
		logThis = QtCore.pyqtSignal(str)
		code = ''

		def __init__(self,REGISTERS):
			super(AppWindow.codeObject, self).__init__()
			self.PORTMAP = {v: k for k, v in REGISTERS.items()} #Lookup port name based on number
			self.compiled = ''
			self.SR = None
			self.GR = None
			self.evalGlobals = {}

			self.evalGlobals['getReg']  = self.getReg
			self.evalGlobals['setReg']  = self.setReg
			self.evalGlobals['print']  = self.printer

		def setCode(self,code,**kwargs):
			try:
				self.compiled = compile(code.encode(), '<string>', mode='exec')
			except SyntaxError as err:
				error_class = err.__class__.__name__
				detail = err.args[0]
				line_number = err.lineno
				return '''<span style="color:red;">%s at line %d : %s</span>''' % (error_class, line_number, detail)
			except Exception as err:
				error_class = err.__class__.__name__
				detail = err.args[0]
				cl, exc, tb = sys.exc_info()
				line_number = traceback.extract_tb(tb)[-1][1]
				return '''<span style="color:red;">%s at line %d: %s</span>''' % (error_class, line_number, detail)

			self.SR = kwargs.get('setReg')
			self.GR = kwargs.get('getReg')
			self.evalGlobals = kwargs
			self.evalGlobals['getReg']  = self.getReg #Overwrite these three. They will be wrapped.
			self.evalGlobals['setReg']  = self.setReg
			self.evalGlobals['print']  = self.printer
			return ''
			

		def printer(self,*args):
			self.logThis.emit('''<span style="color:cyan;">%s</span>'''%(' '.join([str(a) for a in args])))

		def setReg(self,reg,value):
			html=u'''<pre><span>W\u2193</span>{0:s}\t{1:d}\t0x{1:02x} / 0b{1:08b}</pre>'''.format(self.PORTMAP.get(reg,''),value)
			self.logThis.emit(html)
			self.SR(reg,value)

		def getReg(self,reg):
			value = self.GR(reg)
			html=u'''<pre><span>R\u2191</span>{0:s}\t{1:d}\t0x{1:02x} / 0b{1:08b}</pre>'''.format(self.PORTMAP.get(reg,''),value)
			self.logThis.emit(html)
			return value

		def execute(self):
			#old = sys.stdout
			#olde = sys.stderr
			#sys.stdout = self.toLog(self.logThis)
			#sys.stderr = self.toLog(self.logThis)
			try:
				exec(self.compiled,{},self.evalGlobals)
			except SyntaxError as err:
				error_class = err.__class__.__name__
				detail = err.args[0]
				line_number = err.lineno
				self.logThis.emit('''<span style="color:red;">%s at line %d : %s</span>''' % (error_class, line_number, detail))
			except Exception as err:
				error_class = err.__class__.__name__
				detail = err.args[0]
				cl, exc, tb = sys.exc_info()
				line_number = traceback.extract_tb(tb)[-1][1]
				self.logThis.emit('''<span style="color:red;">%s at line %d: %s</span>''' % (error_class, line_number, detail))
			#sys.stdout = old
			#sys.stderr = olde
			self.logThis.emit("Finished executing user code")
			self.finished.emit()
	
	def runCode(self):
		#if self.p:
		#	try:
		#		self.p.fd.read() #Clear junk
		#		self.p.fd.close()
		#	except:pass
		if self.codeThread.isRunning():
			print('one code is already running')
			return

		self.log.clear() #clear the log window
		self.log.setText('''<span style="color:green;">----------User Code Started-----------</span>''')
		kwargs = {}
		for a in dir(self.p):
			attr = getattr(self.p, a)
			if inspect.ismethod(attr) and a[:2]!='__':
				kwargs[a] = attr

		compilemsg = self.codeEval.setCode('{0:s}'.format(self.userCode.toPlainText()),**kwargs)
		if len(compilemsg):
			self.log.append(compilemsg)
			return
		self.codeThread.start()

		self.userCode.setStyleSheet("border: 3px dashed #53ffff;")
		self.tabs.setTabEnabled(0,False)

	def codeFinished(self):
		print('finished')
		self.tabs.setTabEnabled(0,True)
		self.userCode.setStyleSheet("")
		self.uploadingHex = False

	def abort(self):
		if self.codeThread.isRunning():
			self.log.append('''<span style="color:red;">----------Kill Signal(Doesn't work yet. Restart the application)-----------</span>''')
			self.codeThread.quit()
			self.codeThread.terminate()
			del self.codeThread
			self.codeThread = QtCore.QThread()
			self.codeEval = self.codeObject(self.REGISTERS)
			self.codeEval.moveToThread(self.codeThread)
			self.codeEval.finished.connect(self.codeThread.quit)
			self.codeEval.logThis.connect(self.appendLog) #Connect to the log window			
			self.codeThread.started.connect(self.codeEval.execute)
			self.codeThread.finished.connect(self.codeFinished)

	def getReg(self,reg,record = True):
		val = self.p.getReg(reg)
		if record:
			self.updatedRegs[reg] = [0,val]
		return val

	def setReg(self,reg,val,record = True):
		self.p.setReg(reg,val)
		if record:
			self.updatedRegs[reg] = [1,val]
		return val

	def appendLog(self,txt):
		self.log.append(txt)

	def appendLogPlain(self,txt):
		self.log.moveCursor(QtGui.QTextCursor.End)
		self.log.insertPlainText(txt.decode('ascii'))

	def setSerialgauge(self,val):
		self.serialGauge.setValue([val])

	def genLog(self):
		html='''<table border="1" align="center" cellpadding="1" cellspacing="0" style="font-family:arial,helvetica,sans-serif;font-size:9pt;">
		<tbody><tr><td colspan="4">%s</td></tr>'''%(time.ctime())
		#html+='''<tr><td style="background-color:#77cfbb;">R/W</td><td style="background-color:#77cfbb;">REGISTER</td>
		#<td style="background-color:#77cfbb;">Value</td><td style="background-color:#77cfbb;">Hex/Binary</td></tr>'''

		for a in self.updatedRegs:
			row = self.updatedRegs[a]
			html+=u'''
				<tr>
					<td>{0:s}</td>
					<td>{1:s}</td>
					<td>{2:d}</td>
					<td>0b{2:08b} | 0x{2:02x}</td>
				</tr>
				'''.format(u'W \u2193' if row[0] else u'R \u2191',a,row[1])
		html+="</tbody></table>"	
		self.log.setHtml(html)

	def userRegistersAutoRefresh(self,state):
		self.autoUpdateUserRegisters = state

	def updateEverything(self):
		self.locateDevices()
		if not self.checkConnectionStatus():return
		#KuttyPy monitor has handed over control to native code. act as serial monitor/ debug window
		if self.uploadingHex:
			return

		
		if self.userHexRunning:
			t = self.p.fd.read(self.p.fd.in_waiting)
			if len(t):
				self.serialGaugeSignal.emit(t[0])
				self.logThisPlain.emit(t)
			return
		
		#self.setTheme('material')

		if self.codeThread.isRunning():
			return

		if self.autoUpdateUserRegisters:
			for a in range(self.registerList.count()):
				self.registerList.itemWidget(self.registerList.item(a)).execute()
			
		while len(self.commandQ):
			if not self.centralWidget().isEnabled():
				return
			a = self.commandQ.pop(0)
			if a[0] == 'DSTATE': #Digital Out ['DSTATE','Pxx',state]
				pname = 'PORT'+a[1][1].upper()
				bit = int(a[1][2])
				reg = self.getReg(pname)
				reg &=~ (1<<bit)
				if(a[2]):reg |= (1<<bit)
				self.setReg(pname,reg)
			elif a[0] == 'DTYPE': #Digital pin I/O ['DTYPE','Pxx',state]
				pname = 'DDR'+a[1][1].upper()
				bit = int(a[1][2])
				reg = self.getReg(pname)
				reg &=~ (1<<bit)
				if(a[2]):reg |= (1<<bit)
				self.setReg(pname,reg);
			elif a[0] == 'WRITE': #['WRITE','REGNAME',val]
				self.setReg(a[1],a[2]);
			elif a[0] == 'READ': #['READ','REGNAME',function]
				val = self.getReg(a[1])
				a[2](val)
			elif a[0] == 'CNTR1': #['CNTR1',output with setValue function]
				cl = self.getReg('TCNT1L',False)
				ch = self.getReg('TCNT1H',False)
				a[1].setValue(cl|(ch<<8))
			elif a[0] == 'ADC': #['ADC',ADMUX,output with setValue function, to log, or not to log]
				self.setReg('ADMUX',a[1],a[3])
				self.setReg('ADCSRA',196|1,a[3])
				adcl = self.getReg('ADCL',a[3])
				adch = self.getReg('ADCH',a[3])
				a[2].setValue(adcl|(adch<<8))
		
		for a in self.sensorList:
			if a[0].isVisible():
				a[0].setValue(a[0].read())
			
		if self.enableLog.isChecked():
			if self.clearLog.isChecked() and len(self.updatedRegs):
				self.log.clear()
			if len(self.updatedRegs):
				self.genLog()
				self.updatedRegs = OrderedDict()

		if self.pending['status'].ready() and self.monitoring:
			val = self.p.getReg(self.getRegs[self.currentRegister][0])
			self.getRegs[self.currentRegister][1](self.getRegs[self.currentRegister][0],val)
			self.currentRegister +=1
			if self.currentRegister==len(self.getRegs):
				self.currentRegister = 0
			for a in ['B','C','D']:
				self.btns[a].setRegs(self.p.REGSTATES)


		if self.pending['update'].ready():
			pass
			#print('update')

	def updateInputs(self,port,value):
		portchar = port[3]
		for a in range(8):
			name = 'P'+portchar+str(a)
			btn = self.btns.get(name,None)
			if btn is None: continue
			btn.nameIn.setChecked((value>>a)&1)
			if name in self.ADC_PINS: #ADC
				if btn.currentPage == 2: #ADC Page displayed
					self.commandQ.append(['ADC',btn.ADMUX,btn,btn.logstate])
			elif type(btn)==dio.DIOCNTR and btn.currentPage==2: # CNTR
					self.commandQ.append(['CNTR1',btn.slider])

	def newStepperController(self):
		if self.p.connected:
			dialog = dio.DIOSTEPPER(self,total=200,device=self.p)
			dialog.launch()
			self.sensorList.append([dialog,None])


	############ I2C SENSORS #################
	def I2CScan(self):
		if self.p.connected:
			x = self.p.I2CScan()
			print('Responses from: ',x)
			for a in self.sensorList:
				a[0].setParent(None)
				a[1].setParent(None)
			self.sensorList = []
			for a in self.controllerList:
				a[0].setParent(None)
				a[1].setParent(None)
			self.controllerList = []
			for a in x:
				s = self.p.sensors.get(a,None)
				if s is not None:
					btn = QtWidgets.QPushButton(s['name']+':'+hex(a))
					dialog = dio.DIOSENSOR(self,s)
					btn.clicked.connect(dialog.launch)
					self.sensorLayout.addWidget(btn)
					self.sensorList.append([dialog,btn])
					continue

				s = self.p.controllers.get(a,None)
				if s is not None:
					btn = QtWidgets.QPushButton(s['name']+':'+hex(a))
					dialog = dio.DIOCONTROL(self,s)
					btn.clicked.connect(dialog.launch)
					self.sensorLayout.addWidget(btn)
					self.controllerList.append([dialog,btn])
					continue

				s = self.p.special.get(a,None)
				if s is not None:
					btn = QtWidgets.QPushButton(s['name']+':'+hex(a))
					dialog = dio.DIOROBOT(self,s)
					btn.clicked.connect(dialog.launch)
					self.sensorLayout.addWidget(btn)
					self.controllerList.append([dialog,btn])
					continue


	def loadExample(self,filename):
		self.userCode.setPlainText(open(os.path.join(path["examples"],self.EXAMPLES_DIR,filename), "r").read())


	########################### UPLOAD HEX FILE #######################

	class uploadObject(QtCore.QObject):
		finished = QtCore.pyqtSignal()
		logThis = QtCore.pyqtSignal(str)
		logThisPlain = QtCore.pyqtSignal(bytes)
		fname = ''
		p = None
		def __init__(self):
			super(AppWindow.uploadObject, self).__init__()
		def config(self,mode,p,fname):
			self.p = p
			self.fname = fname
			self.mode = mode

		def execute(self):
			if self.mode == 'compileupload':
				try:
					import subprocess
					fname = '.'.join(self.fname.split('.')[:-1])
					cmd = 'avr-gcc -Wall -O2 -mmcu=%s -o "%s" "%s"' %('atmega328p',fname,self.fname)
					self.logThis.emit('''<span style="color:green;">Compiling for Atmega328p (Uno)</span>''')
					print(cmd)
					res = subprocess.getstatusoutput(cmd)
					if res[0] != 0:
						self.logThis.emit('''<span style="color:red;">Compile Error: %s</span>'''%res[1])
						self.finished.emit()
						return

					else:
						self.logThis.emit('''<span style="color:white;">%s</span><br>'''%res[1])
					cmd = 'avr-objcopy -j .text -j .data -O ihex "%s" "%s.hex"' %(fname,fname)
					res = subprocess.getstatusoutput(cmd)
					self.logThis.emit('''<span style="color:white;">%s</span><br>'''%res[1])
					cmd = 'avr-objdump -S "%s" > "%s.lst"'%(fname,fname)
					res = subprocess.getstatusoutput(cmd)
					self.logThis.emit('''<span style="color:white;">%s</span><br>'''%res[1])
					if self.fname[-2:] in ['.c','.C']:
						self.fname = self.fname[:-2]+'.hex' #Replace .c with .hex
						self.mode = 'upload'
						self.logThis.emit('''<span style="color:green;">Generated Hex File</span>''')
					self.logThis.emit('''<span style="color:green;">Finished Compiling: Generated Hex File</span>''')
				except Exception as err:
					self.logThis.emit('''<span style="color:red;">Failed to Compile:%s</span>'''%str(err))

			if self.p.connected:
				if self.mode == 'upload':
					try:
						self.p.fd.setRTS(0);time.sleep(0.01);self.p.fd.setRTS(1);time.sleep(0.4)
						dude = uploader.Uploader(self.p.fd, hexfile=self.fname,logger = self.logThis)
						dude.program()
						dude.verify()
						self.p.fd.setRTS(0);time.sleep(0.01);self.p.fd.setRTS(1);time.sleep(0.2)
						self.p.get_version()
						self.logThis.emit('''<span style="color:green;">Finished upload</span>''')
					except Exception as err:
						self.logThis.emit('''<span style="color:red;">Failed to upload</span>''')
			self.finished.emit()
	
	def uploadHex(self):
		filename = QtWidgets.QFileDialog.getOpenFileName(self," Open a hex file to upload to your KuttyPy", "", "Hex Files (*.hex)")
		if len(filename[0]):
			#self.userCode.setStyleSheet("border: 3px dashed #53ffff;")
			#self.tabs.setTabEnabled(0,False)
			self.uploadingHex = True
			self.log.clear()
			self.log.setText('''<span style="color:cyan;">-- Uploading Code --</span><br>''')
			self.UploadObject.config('upload',self.p,filename[0])
			self.uploadThread.start()


	def openFile(self):
		filename = QtWidgets.QFileDialog.getOpenFileName(self," Open a C file to edit", path["examples"], "C Files (*.c *.C)")
		if len(filename[0]):
			self.filenameLabel.setText(filename[0])
			self.CFile = filename[0]
			self.log.clear()
			self.log.setText('''<span style="color:cyan;">-- Opened File: %s --</span><br>'''%filename[0])
			if 'inux' in platform.system(): #Linux based system
				os.system('%s "%s"' % ('xdg-open', filename[0]))
			else:
				os.system('%s "%s"' % ('open', filename[0]))

	def compileAndUpload(self):
		if self.CFile:
			self.uploadingHex = True
			self.log.clear()
			self.log.setText('''<span style="color:cyan;">-- Compiling and Uploading Code --</span><br>''')
			self.UploadObject.config('compileupload',self.p,self.CFile)
			self.uploadThread.start()
			

	##############################
	def setTheme(self,theme):
		self.setStyleSheet("")
		self.setStyleSheet(open(os.path.join(path["themes"],theme+".qss"), "r").read())

			
	def initializeCommunications(self,port=False):
		if self.p:
			try:self.p.fd.close()
			except:pass
		if port:
			self.p = KuttyPyLibUno.connect(port = port)
		else:
			self.p = KuttyPyLibUno.connect(autoscan=True)
		if self.p.connected:
			self.userApplication.setChecked(False)
			self.setWindowTitle('KuttyPy Interactive Console [{0:s}]'.format(self.p.portname))
			self.updatedRegs=OrderedDict()
			self.currentRegister = 0
			self.getRegs=[
			('PINB',self.updateInputs),
			('PINC',self.updateInputs),
			('PIND',self.updateInputs),
			]


		else:
			self.setWindowTitle('KuttyPy Interactive Console [ Hardware not detected ]')

	def jumpToApplication(self,state):
		if self.p:
			if state:
				self.userHexRunning=True
				self.p.fd.write(b'j') #Skip to application (Bootloader resets) 

				for a in self.docks:
					a.setEnabled(False)
				self.tabs.setEnabled(False)
				self.log.clear()
				self.log.setText('''<span style="color:cyan;">-- Serial Port Monitor --</span><br>''')
				self.serialGauge.show()

			else:
				self.p.fd.setRTS(0)  #Trigger a reset
				time.sleep(0.01)
				self.p.fd.setRTS(1)
				time.sleep(0.1)
				while self.p.fd.in_waiting:
					self.p.fd.read()
				self.p.get_version()
				for a in self.docks:
					a.setEnabled(True)
				self.userHexRunning=False
				self.tabs.setEnabled(True)
				self.serialGauge.hide()
		else:
			if self.isChecked():
				self.setChecked(False)

		
	def makeBottomMenu(self):
		try:self.pushbutton.setParent(None)
		except:pass
		self.pushbutton = QtWidgets.QPushButton('Menu')
		self.pushbutton.setStyleSheet("height: 13px;padding:3px;color: #FFFFFF;")
		menu = QtWidgets.QMenu()

		menu.addAction('Save Window as Svg', self.exportSvg)
		menu.addAction('Open Stepper Controller', self.newStepperController)

		#Theme
		self.themeAction = QtWidgets.QWidgetAction(menu)
		themes = [a.split('.qss')[0] for a in os.listdir(path["themes"]) if '.qss' in a]
		self.themeBox = QtWidgets.QComboBox(); self.themeBox.addItems(themes)
		self.themeBox.currentIndexChanged['QString'].connect(self.setTheme)
		self.themeAction.setDefaultWidget(self.themeBox)
		menu.addAction(self.themeAction)

		self.pushbutton.setMenu(menu)

		self.userApplication = QtWidgets.QCheckBox("User App");
		self.userApplication.toggled['bool'].connect(self.jumpToApplication)
		self.statusBar.addPermanentWidget(self.userApplication)

		self.hexUploadButton = QtWidgets.QPushButton("Upload Hex");
		self.hexUploadButton.clicked.connect(self.uploadHex)
		self.statusBar.addPermanentWidget(self.hexUploadButton)


		self.speedbutton = QtWidgets.QComboBox(); self.speedbutton.addItems(['Slow','Fast','Ultra']);
		self.speedbutton.setCurrentIndex(1);
		self.speedbutton.currentIndexChanged['int'].connect(self.setSpeed)
		self.statusBar.addPermanentWidget(self.speedbutton)

		self.statusBar.addPermanentWidget(self.pushbutton)

	def setSpeed(self,index):
		self.timer.setInterval([100,20,5][index])

	def locateDevices(self):
		try:L = KuttyPyLibUno.getFreePorts(self.p.portname)
		except Exception as e:print(e)
		total = len(L)
		menuChanged = False
		if L != self.shortlist:
			menuChanged = True
			if self.p.connected:
				if self.p.portname not in L:
						self.setWindowTitle('Error : Device Disconnected')
						QtWidgets.QMessageBox.warning(self, 'Connection Error', 'Device Disconnected. Please check the connections')
						try:
							self.p.fd.close()
							self.p.portname = None
						except:pass
						self.p.connected = False
						self.setWindowTitle('KuttyPy Interactive Console [ Hardware not detected ]')

			elif True in L.values():
				reply = QtWidgets.QMessageBox.question(self, 'Connection', 'Device Available. Connect?', QtWidgets.QMessageBox.Yes, QtWidgets.QMessageBox.No)
				if reply == QtWidgets.QMessageBox.Yes:
					self.initializeCommunications()

			#update the shortlist
			self.shortlist=L

	def checkConnectionStatus(self,dialog=False):
		if self.p.connected:return True
		else:
			if dialog: QtWidgets.QMessageBox.warning(self, 'Connection Error', 'Device not connected. Please connect a KuttyPy to the USB port')
			return False
			
	def updateStatus(self):
		if not self.checkConnectionStatus():
			self.countLabel.setText('Not Connected')
			return
		try:
			state,cnt = self.p.getStatus()
			self.currentState = state
			self.countLabel.setText('%s: %d'%("Running" if state else "Paused",cnt))
		except:
			self.countLabel.setText('Disconnect!')
			self.p.fd.close()


	######## WINDOW EXPORT SVG
	def exportSvg(self):
		from utilities.Qt import QtSvg
		path, _filter  = QtWidgets.QFileDialog.getSaveFileName(self, 'Save File', '~/')
		if path:
			generator = QtSvg.QSvgGenerator()
			generator.setFileName(path)
			target_rect = QtCore.QRectF(0, 0, 800, 600)
			generator.setSize(target_rect.size().toSize())#self.size())
			generator.setViewBox(self.rect())
			generator.setTitle("Your title")
			generator.setDescription("some description")
			p = QtGui.QPainter()
			p.begin(generator)
			self.render(p)
			p.end()

	def ipython(self): #Experimental feature. Import ipython and launch console
		if not self.p.connected:
			return
		from utilities import ipy
		if not self.ipy:
			self.ipy = ipy.AppWindow(self,kp = self.p)
		self.ipy.show()
		self.ipy.updateDevice(self.p)

def translators(langDir, lang=None):
	"""
	create a list of translators
	@param langDir a path containing .qm translation
	@param lang the preferred locale, like en_IN.UTF-8, fr_FR.UTF-8, etc.
	@result a list of QtCore.QTranslator instances
	"""
	if lang==None:
			lang=QtCore.QLocale.system().name()
	result=[]
	qtTranslator=QtCore.QTranslator()
	qtTranslator.load("qt_" + lang,
			QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.TranslationsPath))
	result.append(qtTranslator)

	# path to the translation files (.qm files)
	sparkTranslator=QtCore.QTranslator()
	sparkTranslator.load(lang, langDir);
	result.append(sparkTranslator)
	return result

def firstExistingPath(l):
	"""
	Returns the first existing path taken from a list of
	possible paths.
	@param l a list of paths
	@return the first path which exists in the filesystem, or None
	"""
	for p in l:
		if os.path.exists(p):
			return p
	return None

def common_paths():
	"""
	Finds common paths
	@result a dictionary of common paths
	"""
	path={}
	curPath = os.path.dirname(os.path.realpath(__file__))
	path["current"] = curPath
	sharedPath = "/usr/share/kuttypy"
	path["translation"] = firstExistingPath(
			[os.path.join(p, "lang") for p in
			 (curPath, sharedPath,)])
	path["utilities"] = firstExistingPath(
			[os.path.join(p,'utilities') for p in
			 (curPath, sharedPath,)])
	path["templates"] = firstExistingPath(
			[os.path.join(p,'utilities','templates') for p in
			 (curPath, sharedPath,)])

	path["themes"] = firstExistingPath(
			[os.path.join(p,'utilities','themes') for p in
			 (curPath, sharedPath,)])

	path["examples"] = firstExistingPath(
			[os.path.join(p,'examples') for p in
			 (curPath, sharedPath,)])

	path["editor"] = firstExistingPath(
			[os.path.join(p,'editor') for p in
			 (curPath, sharedPath,)])

	lang=str(QtCore.QLocale.system().name()) 
	shortLang=lang[:2]
	return path



def run():
	global path, app, myapp
	path = common_paths()
	app = QtWidgets.QApplication(sys.argv)

	myapp = AppWindow(app=app, path=path)
	myapp.show()
	r = app.exec_()
	'''
	if myapp.p.connected:
		myapp.p.fd.write(b'j')
		#myapp.p.fd.setRTS(0)
		#time.sleep(0.01)
		#myapp.p.fd.setRTS(1)
	'''
	app.deleteLater()
	sys.exit(r)

if __name__ == "__main__":
	run()





# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'dio.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_stack(object):
    def setupUi(self, stack):
        stack.setObjectName("stack")
        stack.resize(270, 27)
        self.inputPage = QtWidgets.QWidget()
        self.inputPage.setObjectName("inputPage")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.inputPage)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.nameIn = QtWidgets.QCheckBox(self.inputPage)
        self.nameIn.setObjectName("nameIn")
        self.horizontalLayout.addWidget(self.nameIn)
        self.pullup = QtWidgets.QCheckBox(self.inputPage)
        self.pullup.setObjectName("pullup")
        self.horizontalLayout.addWidget(self.pullup)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.pushButton = QtWidgets.QPushButton(self.inputPage)
        self.pushButton.setObjectName("pushButton")
        self.horizontalLayout.addWidget(self.pushButton)
        stack.addWidget(self.inputPage)
        self.outputPage = QtWidgets.QWidget()
        self.outputPage.setObjectName("outputPage")
        self.outputLayout = QtWidgets.QHBoxLayout(self.outputPage)
        self.outputLayout.setContentsMargins(0, 0, 0, 0)
        self.outputLayout.setSpacing(0)
        self.outputLayout.setObjectName("outputLayout")
        self.nameOut = QtWidgets.QCheckBox(self.outputPage)
        self.nameOut.setCheckable(True)
        self.nameOut.setChecked(False)
        self.nameOut.setTristate(False)
        self.nameOut.setObjectName("nameOut")
        self.outputLayout.addWidget(self.nameOut)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.outputLayout.addItem(spacerItem1)
        self.pushButton_2 = QtWidgets.QPushButton(self.outputPage)
        self.pushButton_2.setObjectName("pushButton_2")
        self.outputLayout.addWidget(self.pushButton_2)
        stack.addWidget(self.outputPage)

        self.retranslateUi(stack)
        stack.setCurrentIndex(0)
        self.nameOut.clicked['bool'].connect(stack.setOutputState) # type: ignore
        self.pullup.clicked['bool'].connect(stack.setOutputState) # type: ignore
        self.pushButton.clicked.connect(stack.next) # type: ignore
        self.pushButton_2.clicked.connect(stack.next) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(stack)

    def retranslateUi(self, stack):
        _translate = QtCore.QCoreApplication.translate
        stack.setWindowTitle(_translate("stack", "StackedWidget"))
        self.nameIn.setText(_translate("stack", "name"))
        self.pullup.setText(_translate("stack", "Pull-Up"))
        self.pullup.setProperty("class", _translate("stack", "pullup"))
        self.pushButton.setText(_translate("stack", "INPUT"))
        self.nameOut.setText(_translate("stack", "name"))
        self.pushButton_2.setText(_translate("stack", "OUTPUT"))



# Adapted from https://gist.github.com/LegoStormtroopr/6146161

from PyQt5 import QtGui, QtCore, QtWidgets


class myTextEditor(QtWidgets.QPlainTextEdit):
    def __init__(self, parent, codingTabBrowser):
        QtWidgets.QPlainTextEdit.__init__(self, parent)
        self.sourceTab = parent
        self.codingTabs = codingTabBrowser
        self.textChanged.connect(self.textChange)
        self.undoAvailable['bool'].connect(self.undoStatus)
        self.changed = False

    def markAsSaved(self, state):
        self.changed = not state

    def undoStatus(self, s):
        if s == False:
            fname = self.codingTabs.tabText(self.codingTabs.indexOf(self.sourceTab))
            if (fname[-1] == '*'):
                self.codingTabs.setTabText(self.codingTabs.indexOf(self.sourceTab), fname[:-1])
                self.changed = False

    def textChange(self):
        fname = self.codingTabs.tabText(self.codingTabs.indexOf(self.sourceTab))
        if (fname[-1] != '*'):
            self.changed = True
            self.codingTabs.setTabText(self.codingTabs.indexOf(self.sourceTab), fname + '*')

    def keyPressEvent(self, event):
        # Shift + Tab is not the same as trying to catch a Shift modifier and a tab Key.
        # Shift + Tab is a Backtab!!
        cursor = self.textCursor()

        if event.key() == QtCore.Qt.Key.Key_Tab:
            if cursor.selectionStart() == cursor.selectionEnd():
                return QtWidgets.QPlainTextEdit.keyPressEvent(self, event)

            tab = "\t"
            start = cursor.selectionStart()
            end = cursor.selectionEnd()
            cursor.setPosition(end)
            cursor.movePosition(cursor.EndOfLine)
            end = cursor.position()
            cursor.setPosition(start)
            cursor.movePosition(cursor.StartOfLine)
            start = cursor.position()
            while cursor.position() < end:
                cursor.movePosition(cursor.StartOfLine)
                cursor.insertText(tab)
                # end += tab.count()
                cursor.movePosition(cursor.EndOfLine)
                cursor.movePosition(QtGui.QTextCursor.Right, 1)

        elif event.key() == QtCore.Qt.Key.Key_Backtab:
            self.unindentSelectedBlock()
            return

            # Copy the current selection
            pos = cursor.position()  # Where a selection ends
            anchor = cursor.anchor()  # Where a selection starts (can be the same as above)
            # Can put QtGui.QTextCursor.MoveAnchor as the 2nd arg, but this is the default
            cursor.setPosition(pos)
            # Move the position back one, selection the character prior to the original position
            cursor.setPosition(pos - 1, QtGui.QTextCursor.KeepAnchor)

            if str(cursor.selectedText()) == "\t":
                # The prior character is a tab, so delete the selection
                cursor.removeSelectedText()
                # Reposition the cursor with the one character offset
                cursor.setPosition(anchor - 1)
                cursor.setPosition(pos - 1, QtGui.QTextCursor.KeepAnchor)
            else:
                # Try all of the above, looking before the anchor (This helps if the achor is before a tab)
                cursor.setPosition(anchor)
                cursor.setPosition(anchor - 1, QtGui.QTextCursor.KeepAnchor)
                if str(cursor.selectedText()) == "\t":
                    self.unindentSelectedBlock()
                    return
                    cursor.removeSelectedText()
                    cursor.setPosition(anchor - 1)
                    cursor.setPosition(pos - 1, QtGui.QTextCursor.KeepAnchor)
                else:
                    # Its not a tab, so reset the selection to what it was
                    cursor.setPosition(anchor)
                    cursor.setPosition(pos, QtGui.QTextCursor.KeepAnchor)
        else:
            return QtWidgets.QPlainTextEdit.keyPressEvent(self, event)

    def unindentSelectedBlock(self):
        cursor = self.textCursor()
        # Copy the current selection

        # Save the current cursor position
        original_position = cursor.position()

        # Get the selected text
        selected_text = cursor.selectedText()
        # Perform replacement on the selected text
        modified_text = selected_text.replace('\u2029\t', '\u2029')
        if modified_text.startswith('\t'):
            print('remove starting tab')
            modified_text=modified_text[1:]

        #print('unindent', selected_text, modified_text)
        # Replace the original selected block with the modified text
        cursor.removeSelectedText()
        cursor.insertText(modified_text)

        # Restore the cursor position
        cursor.setPosition(original_position)

        # Set the cursor position to the updated position
        self.setTextCursor(cursor)


# Read ADC Values and plot them
import time
#from kuttyPy import *
from matplotlib import pyplot as plt

setReg('ADMUX', (1<<6) | 0b01011) #REF_AVCC | A1-A0 x200
for a in range(50): 
    setReg('ADCSRA', 196)
    cl = getReg('ADCL')
    ch = getReg('ADCH')
    plt.scatter(a, (ch<<8)|cl ,s=5)
    plt.pause(0.01) #Wait 10 mS
